package java.util.concurrent;

import java.lang.reflect.Array;
import java.util.Arrays;
import java.util.Spliterator;
import java.util.NoSuchElementException;
import java.util.concurrent.locks.LockSupport;
import sun.misc.Contended;
import java.util.concurrent.locks.ReentrantLock;
import java.util.function.ToIntFunction;
import java.util.function.ToLongFunction;
import java.util.function.ToDoubleFunction;
import java.util.function.IntBinaryOperator;
import java.util.function.ToIntBiFunction;
import java.util.function.LongBinaryOperator;
import java.util.function.ToLongBiFunction;
import java.util.function.DoubleBinaryOperator;
import java.util.function.ToDoubleBiFunction;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Consumer;
import java.util.Enumeration;
import java.util.function.Function;
import java.util.function.BiFunction;
import java.util.function.BiConsumer;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.Set;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.lang.reflect.Type;
import java.lang.reflect.ParameterizedType;
import sun.misc.Unsafe;
import java.io.ObjectStreamField;
import java.io.Serializable;
import java.util.AbstractMap;

public class ConcurrentHashMap<K, V> extends AbstractMap<K, V> implements ConcurrentMap<K, V>, Serializable
{
    private static final long serialVersionUID = 7249069246763182397L;
    private static final int MAXIMUM_CAPACITY = 1073741824;
    private static final int DEFAULT_CAPACITY = 16;
    static final int MAX_ARRAY_SIZE = 2147483639;
    private static final int DEFAULT_CONCURRENCY_LEVEL = 16;
    private static final float LOAD_FACTOR = 0.75f;
    static final int TREEIFY_THRESHOLD = 8;
    static final int UNTREEIFY_THRESHOLD = 6;
    static final int MIN_TREEIFY_CAPACITY = 64;
    private static final int MIN_TRANSFER_STRIDE = 16;
    private static int RESIZE_STAMP_BITS;
    private static final int MAX_RESIZERS;
    private static final int RESIZE_STAMP_SHIFT;
    static final int MOVED = -1;
    static final int TREEBIN = -2;
    static final int RESERVED = -3;
    static final int HASH_BITS = Integer.MAX_VALUE;
    static final int NCPU;
    private static final ObjectStreamField[] serialPersistentFields;
    transient volatile ConcurrentHashMap.Node<K, V>[] table;
    private transient volatile ConcurrentHashMap.Node<K, V>[] nextTable;
    private transient volatile long baseCount;
    private transient volatile int sizeCtl;
    private transient volatile int transferIndex;
    private transient volatile int cellsBusy;
    private transient volatile ConcurrentHashMap.CounterCell[] counterCells;
    private transient ConcurrentHashMap.KeySetView<K, V> keySet;
    private transient ConcurrentHashMap.ValuesView<K, V> values;
    private transient ConcurrentHashMap.EntrySetView<K, V> entrySet;
    private static final Unsafe U;
    private static final long SIZECTL;
    private static final long TRANSFERINDEX;
    private static final long BASECOUNT;
    private static final long CELLSBUSY;
    private static final long CELLVALUE;
    private static final long ABASE;
    private static final int ASHIFT;
    
    static final int spread(int n) {
        return (n ^ n >>> 16) & Integer.MAX_VALUE;
    }
    
    private static final int tableSizeFor(int n) {
        int n2 = n - 1;
        int n3 = n2 | n2 >>> 1;
        int n4 = n3 | n3 >>> 2;
        int n5 = n4 | n4 >>> 4;
        int n6 = n5 | n5 >>> 8;
        int n7 = n6 | n6 >>> 16;
        return (n7 < 0) ? 1 : ((n7 >= 1073741824) ? 1073741824 : (n7 + 1));
    }
    
    static Class<?> comparableClassFor(Object o) {
        if ((o instanceof Comparable)) {
            Class<Object> class1;
            if ((class1 = o.getClass()) == String.class) {
                return class1;
            }
            Type[] genericInterfaces;
            if ((genericInterfaces = class1.getGenericInterfaces()) != null) {
                int i = 0;
                while (i < genericInterfaces.length) {
                    Type type;
                    ParameterizedType parameterizedType;
                    Type[] actualTypeArguments;
                    if ((type = genericInterfaces[i]) instanceof ParameterizedType && (parameterizedType = (ParameterizedType)type).getRawType() == Comparable.class && (actualTypeArguments = parameterizedType.getActualTypeArguments()) != null && actualTypeArguments.length == 1 && actualTypeArguments[0] == class1) {
                        return class1;
                    }
                    ++i;
                }
            }
        }
        return null;
    }
    
    static int compareComparables(Class<?> clazz, Object o, Object o2) {
        return (o2 == null || o2.getClass() != clazz) ? 0 : ((Comparable)o).compareTo(o2);
    }
    
    static final <K, V> ConcurrentHashMap.Node<K, V> tabAt(ConcurrentHashMap.Node<K, V>[] o, int n) {
        return (ConcurrentHashMap.Node)ConcurrentHashMap.U.getObjectVolatile(o, ((long)n << ConcurrentHashMap.ASHIFT) + ConcurrentHashMap.ABASE);
    }
    
    static final <K, V> boolean casTabAt(ConcurrentHashMap.Node<K, V>[] o, int n, ConcurrentHashMap.Node<K, V> expected, ConcurrentHashMap.Node<K, V> x) {
        return ConcurrentHashMap.U.compareAndSwapObject(o, ((long)n << ConcurrentHashMap.ASHIFT) + ConcurrentHashMap.ABASE, expected, x);
    }
    
    static final <K, V> void setTabAt(ConcurrentHashMap.Node<K, V>[] o, int n, ConcurrentHashMap.Node<K, V> x) {
        ConcurrentHashMap.U.putObjectVolatile(o, ((long)n << ConcurrentHashMap.ASHIFT) + ConcurrentHashMap.ABASE, x);
    }
    
    public ConcurrentHashMap() {
    }
    
    public ConcurrentHashMap(int n) {
        if (n < 0) {
            throw new IllegalArgumentException();
        }
        this.sizeCtl = ((n >= 536870912) ? 1073741824 : tableSizeFor(n + (n >>> 1) + 1));
        return;
    }
    
    public ConcurrentHashMap(Map<? extends K, ? extends V> map) {
        this.sizeCtl = 16;
        this.putAll(map);
    }
    
    public ConcurrentHashMap(int n, float n2) {
        this(n, n2, 1);
    }
    
    public ConcurrentHashMap(int n, float n2, int n3) {
        if (n2 <= 0.0f || n < 0 || n3 <= 0) {
            throw new IllegalArgumentException();
        }
        if (n < n3) {
            n = n3;
        }
        long n4 = (long)(1.0 + (double)((float)(long)n / n2));
        this.sizeCtl = ((n4 >= 1073741824L) ? 1073741824 : tableSizeFor((int)n4));
        return;
    }
    
    @Override
    public int size() {
        long sumCount = this.sumCount();
        return (sumCount < 0L) ? 0 : ((sumCount > 2147483647L) ? Integer.MAX_VALUE : ((int)sumCount));
    }
    
    @Override
    public boolean isEmpty() {
        return this.sumCount() <= 0L;
    }
    
    @Override
    public V get(Object o) {
        int spread = spread(o.hashCode());
        ConcurrentHashMap.Node[] table;
        int length;
        ConcurrentHashMap.Node<K, V> node;
        if ((table = this.table) != null && (length = table.length) > 0 && (node = tabAt(table, length - 1 & spread)) != null) {
            int hash;
            if ((hash = node.hash) == spread) {
                Object key;
                if ((key = node.key) == o || (key != null && o.equals(key))) {
                    return node.val;
                }
            }
            else if (hash < 0) {
                ConcurrentHashMap.Node<Object, Object> find;
                return ((find = node.find(spread, o)) != null) ? find.val : null;
            }
            while ((node = node.next) != null) {
                Object key2;
                if (node.hash != spread || ((key2 = node.key) != o && (key2 == null || !o.equals(key2)))) {
                    continue;
                }
                return node.val;
            }
        }
        return null;
    }
    
    @Override
    public boolean containsKey(Object o) {
        return this.get(o) != null;
    }
    
    @Override
    public boolean containsValue(Object o) {
        if (o == null) {
            throw new NullPointerException();
        }
        ConcurrentHashMap.Node[] table;
        if ((table = this.table) != null) {
            ConcurrentHashMap.Node<Object, Object> advance;
            while ((advance = new ConcurrentHashMap.Traverser(table, table.length, 0, table.length).advance()) != null) {
                Object val;
                if ((val = advance.val) == o || (val != null && o.equals(val))) {
                    return true;
                }
                continue;
            }
        }
        return false;
    }
    
    @Override
    public V put(K k, V v) {
        return this.putVal(k, v, false);
    }
    
    final V putVal(K k, V v, boolean b) {
        if (k == null || v == null) {
            throw new NullPointerException();
        }
        int spread = spread(k.hashCode());
        int n = 0;
        ConcurrentHashMap.Node[] array = this.table;
        while (true) {
            int length;
            if (array == null || (length = array.length) == 0) {
                array = this.initTable();
            }
            else {
                int n2;
                ConcurrentHashMap.Node<K, V> node;
                if ((node = tabAt(array, n2 = (length - 1 & spread))) == null) {
                    if (casTabAt(array, n2, null, new ConcurrentHashMap.Node(spread, k, v))) {
                        break;
                    }
                    continue;
                }
                else {
                    int hash;
                    if ((hash = node.hash) == -1) {
                        array = this.helpTransfer(array, node);
                    }
                    else {
                        Object o = null;
                        synchronized (node) {
                            Label_0308: {
                                if (tabAt(array, n2) == node) {
                                    if (hash >= 0) {
                                        n = 1;
                                        Object key;
                                        while (node.hash != spread || ((key = node.key) != k && (key == null || !k.equals(key)))) {
                                            if ((node = node.next) == null) {
                                                node.next = new ConcurrentHashMap.Node<Object, Object>(spread, k, v);
                                                break Label_0308;
                                            }
                                            ++n;
                                        }
                                        o = node.val;
                                        if (!b) {
                                            node.val = v;
                                        }
                                    }
                                    else if ((node instanceof ConcurrentHashMap.TreeBin)) {
                                        n = 2;
                                        ConcurrentHashMap.TreeNode<Object, Object> putTreeVal;
                                        if ((putTreeVal = ((ConcurrentHashMap.TreeBin)node).putTreeVal(spread, k, v)) != null) {
                                            o = putTreeVal.val;
                                            if (!b) {
                                                putTreeVal.val = v;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        if (n != 0) {
                            if (n >= 8) {
                                this.treeifyBin(array, n2);
                            }
                            if (o != null) {
                                return o;
                            }
                            break;
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
        }
        this.addCount(1L, n);
        return null;
    }
    
    @Override
    public void putAll(Map<? extends K, ? extends V> map) {
        this.tryPresize(map.size());
        for (Map.Entry entry : map.entrySet()) {
            this.putVal(entry.getKey(), entry.getValue(), false);
        }
    }
    
    @Override
    public V remove(Object o) {
        return this.replaceNode(o, null, null);
    }
    
    final V replaceNode(Object o, V v, Object o2) {
        int spread = spread(o.hashCode());
        ConcurrentHashMap.Node[] array = this.table;
        int length;
        while (array != null && (length = array.length) != 0) {
            int n;
            ConcurrentHashMap.Node<K, V> node;
            if ((node = tabAt(array, n = (length - 1 & spread))) == null) {
                break;
            }
            int hash;
            if ((hash = node.hash) == -1) {
                array = this.helpTransfer(array, node);
            }
            else {
                Object o3 = null;
                boolean n2 = false;
                synchronized (node) {
                    Label_0372: {
                        if (tabAt(array, n) == node) {
                            if (hash >= 0) {
                                n2 = true;
                                ConcurrentHashMap.Node<Object, Object> node2 = null;
                                Object key;
                                while (node.hash != spread || ((key = node.key) != o && (key == null || !o.equals(key)))) {
                                    node2 = node;
                                    if ((node = node.next) != null) {
                                        continue;
                                    }
                                    break Label_0372;
                                }
                                Object val2 = node.val;
                                if (o2 == null || o2 == val2 || (val2 != null && o2.equals(val2))) {
                                    o3 = val2;
                                    if (v != null) {
                                        node.val = v;
                                    }
                                    else if (node2 != null) {
                                        node2.next = node.next;
                                    }
                                    else {
                                        setTabAt(array, n, node.next);
                                    }
                                }
                            }
                            else if ((node instanceof ConcurrentHashMap.TreeBin)) {
                                n2 = true;
                                ConcurrentHashMap.TreeBin<Object, Object> treeBin = (ConcurrentHashMap.TreeBin)node;
                                ConcurrentHashMap.TreeNode<Object, Object> root;
                                ConcurrentHashMap.TreeNode<Object, Object> treeNode;
                                if ((root = treeBin.root) != null && (treeNode = root.findTreeNode(spread, o, null)) != null) {
                                    Object val = treeNode.val;
                                    if (o2 == null || o2 == val || (val != null && o2.equals(val))) {
                                        o3 = val;
                                        if (v != null) {
                                            treeNode.val = v;
                                        }
                                        else if (treeBin.removeTreeNode(treeNode)) {
                                            setTabAt(array, n, untreeify(treeBin.first));
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                if (!(n2 == 0)) {
                    if (o3 != null) {
                        if (v == null) {
                            this.addCount(-1L, -1);
                            return o3;
                        }
                        return o3;
                    }
                    else {
                        break;
                    }
                }
                else {
                    continue;
                }
            }
        }
        return null;
    }
    
    @Override
    public void clear() {
        long n = 0L;
        int n2 = 0;
        ConcurrentHashMap.Node[] array = this.table;
        while (array != null && n2 < array.length) {
            ConcurrentHashMap.Node<K, V> tab = tabAt(array, n2);
            if (tab == null) {
                ++n2;
            }
            else {
                int hash;
                if ((hash = tab.hash) == -1) {
                    array = this.helpTransfer(array, tab);
                    n2 = 0;
                }
                else {
                    synchronized (tab) {
                        if (tabAt(array, n2) == tab) {
                            ConcurrentHashMap.TreeNode<K, V> treeNode = (hash >= 0) ? tab : ((tab instanceof ConcurrentHashMap.TreeBin) ? ((ConcurrentHashMap.TreeBin)tab).first : null);
                            while (tab != null) {
                                --n;
                                ConcurrentHashMap.Node<Object, Object> next = tab.next;
                            }
                            setTabAt(array, n2++, null);
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
        }
        if (n != 0L) {
            this.addCount(n, -1);
        }
    }
    
    @Override
    public ConcurrentHashMap.KeySetView<K, V> keySet() {
        ConcurrentHashMap.KeySetView<Object, Object> keySet;
        return ((keySet = this.keySet) != null) ? keySet : (this.keySet = new ConcurrentHashMap.KeySetView(this, null));
    }
    
    @Override
    public Collection<V> values() {
        ConcurrentHashMap.ValuesView<Object, Object> values;
        return ((values = this.values) != null) ? values : (this.values = new ConcurrentHashMap.ValuesView(this));
    }
    
    @Override
    public Set<Map.Entry<K, V>> entrySet() {
        ConcurrentHashMap.EntrySetView<Object, Object> entrySet;
        return ((entrySet = this.entrySet) != null) ? entrySet : (this.entrySet = new ConcurrentHashMap.EntrySetView(this));
    }
    
    @Override
    public int hashCode() {
        int n = 0;
        ConcurrentHashMap.Node[] table;
        if ((table = this.table) != null) {
            ConcurrentHashMap.Node<Object, Object> advance;
            while ((advance = new ConcurrentHashMap.Traverser(table, table.length, 0, table.length).advance()) != null) {
                n += (advance.key.hashCode() ^ advance.val.hashCode());
            }
        }
        return n;
    }
    
    @Override
    public String toString() {
        ConcurrentHashMap.Node[] table;
        int n = ((table = this.table) == null) ? 0 : table.length;
        ConcurrentHashMap.Traverser<Object, Object> traverser = new ConcurrentHashMap.Traverser(table, n, 0, n);
        StringBuilder sb = new StringBuilder();
        sb.append('{');
        ConcurrentHashMap.Node<Object, Object> node;
        if ((node = traverser.advance()) != null) {
            while (true) {
                Object key = node.key;
                Object val = node.val;
                sb.append((key == this) ? "(this Map)" : key);
                sb.append('=');
                sb.append((val == this) ? "(this Map)" : val);
                if ((node = traverser.advance()) == null) {
                    break;
                }
                sb.append(',').append(' ');
            }
        }
        return sb.append('}').toString();
    }
    
    @Override
    public boolean equals(Object o) {
        if (o != this) {
            if (!(o instanceof Map)) {
                return false;
            }
            Map<Object, Object> map = (Map)o;
            ConcurrentHashMap.Node[] table;
            int n = ((table = this.table) == null) ? 0 : table.length;
            ConcurrentHashMap.Node<Object, Object> advance;
            while ((advance = new ConcurrentHashMap.Traverser(table, n, 0, n).advance()) != null) {
                Object val = advance.val;
                Object value = map.get(advance.key);
                if (value == null || (value != val && !value.equals(val))) {
                    return false;
                }
                continue;
            }
            for (Map.Entry entry : map.entrySet()) {
                Object key;
                Object value2;
                Object value3;
                if ((key = entry.getKey()) == null || (value2 = entry.getValue()) == null || (value3 = this.get(key)) == null || (value2 != value3 && !value2.equals(value3))) {
                    return false;
                }
                continue;
            }
        }
        return true;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        int n = 0;
        int i;
        for (i = 1; i < 16; i <<= 1) {
            ++n;
        }
        int n2 = 32 - n;
        int n3 = i - 1;
        ConcurrentHashMap.Segment[] array = (ConcurrentHashMap.Segment[])new ConcurrentHashMap.Segment[16];
        for (int j = 0; j < array.length; ++j) {
            array[j] = new ConcurrentHashMap.Segment(0.75f);
        }
        objectOutputStream.putFields().put("segments", array);
        objectOutputStream.putFields().put("segmentShift", n2);
        objectOutputStream.putFields().put("segmentMask", n3);
        objectOutputStream.writeFields();
        ConcurrentHashMap.Node[] table;
        if ((table = this.table) != null) {
            ConcurrentHashMap.Node<Object, Object> advance;
            while ((advance = new ConcurrentHashMap.Traverser(table, table.length, 0, table.length).advance()) != null) {
                objectOutputStream.writeObject(advance.key);
                objectOutputStream.writeObject(advance.val);
            }
        }
        objectOutputStream.writeObject(null);
        objectOutputStream.writeObject(null);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        this.sizeCtl = -1;
        objectInputStream.defaultReadObject();
        long n = 0L;
        ConcurrentHashMap.Node next = null;
        while (true) {
            Object object = objectInputStream.readObject();
            Object object2 = objectInputStream.readObject();
            if (object != null && object2 != null) {
                next = new ConcurrentHashMap.Node(spread(object.hashCode()), object, object2, next);
                ++n;
            }
            else {
                break;
            }
        }
        if (n == 0L) {
            this.sizeCtl = 0;
        }
        else {
            int tableSize;
            if (n >= 536870912L) {
                tableSize = 1073741824;
            }
            else {
                int n2 = (int)n;
                tableSize = tableSizeFor(n2 + (n2 >>> 1) + 1);
            }
            ConcurrentHashMap.Node[] table = (ConcurrentHashMap.Node[])new ConcurrentHashMap.Node[tableSize];
            int n3 = tableSize - 1;
            long baseCount = 0L;
            while (next != null) {
                ConcurrentHashMap.Node<Object, Object> next2 = next.next;
                int hash = next.hash;
                int n4 = hash & n3;
                ConcurrentHashMap.Node<K, V> node;
                int n6;
                if ((node = tabAt(table, n4)) == null) {
                    n6 = 1;
                }
                else {
                    Object key = next.key;
                    if (node.hash < 0) {
                        if (((ConcurrentHashMap.TreeBin)node).putTreeVal(hash, key, next.val) == null) {
                            ++baseCount;
                        }
                        n6 = 0;
                    }
                    else {
                        int n5 = 0;
                        n6 = 1;
                        while (node != null) {
                            Object key2;
                            if (node.hash == hash && ((key2 = node.key) == key || (key2 != null && key.equals(key2)))) {
                                n6 = 0;
                                break;
                            }
                            ++n5;
                            node = node.next;
                        }
                        if (n6 != 0 && n5 >= 8) {
                            n6 = 0;
                            ++baseCount;
                            next.next = node;
                            Object o = null;
                            Object prev = null;
                            Object next3;
                            for (; next != null; prev = next3, next = next.next) {
                                next3 = new ConcurrentHashMap.TreeNode(next.hash, next.key, next.val);
                                if ((next3.prev = prev) == null) {
                                    o = next3;
                                }
                                else {
                                    prev.next = next3;
                                }
                            }
                            setTabAt(table, n4, new ConcurrentHashMap.TreeBin(o));
                        }
                    }
                }
                if (n6 != 0) {
                    ++baseCount;
                    next.next = node;
                    setTabAt(table, n4, next);
                }
                next = next2;
            }
            this.table = table;
            this.sizeCtl = tableSize - (tableSize >>> 2);
            this.baseCount = baseCount;
        }
    }
    
    @Override
    public V putIfAbsent(K k, V v) {
        return this.putVal(k, v, true);
    }
    
    @Override
    public boolean remove(Object o, Object o2) {
        if (o == null) {
            throw new NullPointerException();
        }
        return o2 != null && this.replaceNode(o, null, o2) != null;
    }
    
    @Override
    public boolean replace(K k, V v, V v2) {
        if (k == null || v == null || v2 == null) {
            throw new NullPointerException();
        }
        return this.replaceNode(k, v2, v) != null;
    }
    
    @Override
    public V replace(K k, V v) {
        if (k == null || v == null) {
            throw new NullPointerException();
        }
        return this.replaceNode(k, v, null);
    }
    
    @Override
    public V getOrDefault(Object o, V v) {
        Object value;
        return ((value = this.get(o)) == null) ? v : value;
    }
    
    @Override
    public void forEach(BiConsumer<? super K, ? super V> biConsumer) {
        if (biConsumer == null) {
            throw new NullPointerException();
        }
        ConcurrentHashMap.Node[] table;
        if ((table = this.table) != null) {
            ConcurrentHashMap.Node<Object, Object> advance;
            while ((advance = new ConcurrentHashMap.Traverser(table, table.length, 0, table.length).advance()) != null) {
                biConsumer.accept(advance.key, advance.val);
            }
        }
        return;
    }
    
    @Override
    public void replaceAll(BiFunction<? super K, ? super V, ? extends V> biFunction) {
        if (biFunction == null) {
            throw new NullPointerException();
        }
        ConcurrentHashMap.Node[] table;
        if ((table = this.table) != null) {
            ConcurrentHashMap.Node<Object, Object> advance;
            while ((advance = new ConcurrentHashMap.Traverser(table, table.length, 0, table.length).advance()) != null) {
                Object o = advance.val;
                Object key = advance.key;
                while (true) {
                    Object apply = biFunction.apply(key, o);
                    if (apply == null) {
                        throw new NullPointerException();
                    }
                    if (this.replaceNode(key, apply, o) == null && (o = this.get(key)) != null) {
                        continue;
                    }
                    break;
                }
            }
        }
        return;
    }
    
    @Override
    public V computeIfAbsent(K k, Function<? super K, ? extends V> function) {
        if (k == null || function == null) {
            throw new NullPointerException();
        }
        int spread = spread(k.hashCode());
        Object o = null;
        int n = 0;
        ConcurrentHashMap.Node[] array = this.table;
        while (true) {
            int length;
            if (array == null || (length = array.length) == 0) {
                array = this.initTable();
            }
            else {
                int n2;
                ConcurrentHashMap.Node<K, V> node;
                if ((node = tabAt(array, n2 = (length - 1 & spread))) == null) {
                    ConcurrentHashMap.ReservationNode o2 = new ConcurrentHashMap.ReservationNode();
                    synchronized (o2) {
                        if (casTabAt(array, n2, null, o2)) {
                            n = 1;
                            Object o3 = null;
                            try {
                                if ((o = function.apply(k)) != null) {
                                    o3 = new ConcurrentHashMap.Node(spread, k, o);
                                }
                            }
                            finally {
                                setTabAt(array, n2, o3);
                            }
                        }
                    }
                    if (n != 0) {
                        break;
                    }
                    continue;
                }
                else {
                    int hash;
                    if ((hash = node.hash) == -1) {
                        array = this.helpTransfer(array, node);
                    }
                    else {
                        boolean n3 = false;
                        synchronized (node) {
                            Label_0433: {
                                if (tabAt(array, n2) == node) {
                                    if (hash >= 0) {
                                        n = 1;
                                        Object key;
                                        while (node.hash != spread || ((key = node.key) != k && (key == null || !k.equals(key)))) {
                                            if ((node = node.next) == null) {
                                                if ((o = function.apply(k)) != null) {
                                                    n3 = true;
                                                    node.next = new ConcurrentHashMap.Node<Object, Object>(spread, k, o);
                                                }
                                                break Label_0433;
                                            }
                                            else {
                                                ++n;
                                            }
                                        }
                                        o = node.val;
                                    }
                                    else if ((node instanceof ConcurrentHashMap.TreeBin)) {
                                        n = 2;
                                        ConcurrentHashMap.TreeBin treeBin = (ConcurrentHashMap.TreeBin)node;
                                        ConcurrentHashMap.TreeNode<Object, Object> root;
                                        ConcurrentHashMap.TreeNode<Object, Object> treeNode;
                                        if ((root = treeBin.root) != null && (treeNode = root.findTreeNode(spread, k, null)) != null) {
                                            o = treeNode.val;
                                        }
                                        else if ((o = function.apply(k)) != null) {
                                            n3 = true;
                                            treeBin.putTreeVal(spread, k, o);
                                        }
                                    }
                                }
                            }
                        }
                        if (n != 0) {
                            if (n >= 8) {
                                this.treeifyBin(array, n2);
                            }
                            if (n3 == 0) {
                                return o;
                            }
                            break;
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
        }
        if (o != null) {
            this.addCount(1L, n);
            return o;
        }
        return o;
    }
    
    @Override
    public V computeIfPresent(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
        if (k == null || biFunction == null) {
            throw new NullPointerException();
        }
        int spread = spread(k.hashCode());
        Object o = null;
        int n = 0;
        int n2 = 0;
        ConcurrentHashMap.Node[] array = this.table;
        while (true) {
            int length;
            if (array == null || (length = array.length) == 0) {
                array = this.initTable();
            }
            else {
                int n3;
                ConcurrentHashMap.Node<K, V> node;
                if ((node = tabAt(array, n3 = (length - 1 & spread))) == null) {
                    break;
                }
                int hash;
                if ((hash = node.hash) == -1) {
                    array = this.helpTransfer(array, node);
                }
                else {
                    synchronized (node) {
                        Label_0371: {
                            if (tabAt(array, n3) == node) {
                                if (hash >= 0) {
                                    n2 = 1;
                                    ConcurrentHashMap.Node<Object, Object> node2 = null;
                                    Object key;
                                    while (node.hash != spread || ((key = node.key) != k && (key == null || !k.equals(key)))) {
                                        node2 = node;
                                        if ((node = node.next) == null) {
                                            break Label_0371;
                                        }
                                        ++n2;
                                    }
                                    o = biFunction.apply(k, node.val);
                                    if (o != null) {
                                        node.val = o;
                                    }
                                    else {
                                        n = -1;
                                        ConcurrentHashMap.Node<K, V> next = node.next;
                                        if (node2 != null) {
                                            node2.next = next;
                                        }
                                        else {
                                            setTabAt(array, n3, next);
                                        }
                                    }
                                }
                                else if ((node instanceof ConcurrentHashMap.TreeBin)) {
                                    n2 = 2;
                                    ConcurrentHashMap.TreeBin<Object, Object> treeBin = (ConcurrentHashMap.TreeBin)node;
                                    ConcurrentHashMap.TreeNode<Object, Object> root;
                                    ConcurrentHashMap.TreeNode<Object, Object> treeNode;
                                    if ((root = treeBin.root) != null && (treeNode = root.findTreeNode(spread, k, null)) != null) {
                                        o = biFunction.apply(k, treeNode.val);
                                        if (o != null) {
                                            treeNode.val = o;
                                        }
                                        else {
                                            n = -1;
                                            if (treeBin.removeTreeNode(treeNode)) {
                                                setTabAt(array, n3, untreeify(treeBin.first));
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    if (n2 != 0) {
                        break;
                    }
                    continue;
                }
            }
        }
        if (n != 0) {
            this.addCount((long)n, n2);
            return o;
        }
        return o;
    }
    
    @Override
    public V compute(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
        if (k == null || biFunction == null) {
            throw new NullPointerException();
        }
        int spread = spread(k.hashCode());
        Object o = null;
        int n = 0;
        int n2 = 0;
        ConcurrentHashMap.Node[] array = this.table;
        while (true) {
            int length;
            if (array == null || (length = array.length) == 0) {
                array = this.initTable();
            }
            else {
                int n3;
                ConcurrentHashMap.Node<K, V> node;
                if ((node = tabAt(array, n3 = (length - 1 & spread))) == null) {
                    ConcurrentHashMap.ReservationNode o2 = new ConcurrentHashMap.ReservationNode();
                    synchronized (o2) {
                        if (casTabAt(array, n3, null, o2)) {
                            n2 = 1;
                            Object o3 = null;
                            try {
                                if ((o = biFunction.apply(k, null)) != null) {
                                    n = 1;
                                    o3 = new ConcurrentHashMap.Node(spread, k, o);
                                }
                            }
                            finally {
                                setTabAt(array, n3, o3);
                            }
                        }
                    }
                    if (n2 != 0) {
                        break;
                    }
                    continue;
                }
                else {
                    int hash;
                    if ((hash = node.hash) == -1) {
                        array = this.helpTransfer(array, node);
                    }
                    else {
                        synchronized (node) {
                            Label_0560: {
                                if (tabAt(array, n3) == node) {
                                    if (hash >= 0) {
                                        n2 = 1;
                                        ConcurrentHashMap.Node<Object, Object> node2 = null;
                                        Object key;
                                        while (node.hash != spread || ((key = node.key) != k && (key == null || !k.equals(key)))) {
                                            node2 = node;
                                            if ((node = node.next) == null) {
                                                o = biFunction.apply(k, null);
                                                if (o != null) {
                                                    n = 1;
                                                    node.next = new ConcurrentHashMap.Node<Object, Object>(spread, k, o);
                                                }
                                                break Label_0560;
                                            }
                                            else {
                                                ++n2;
                                            }
                                        }
                                        o = biFunction.apply(k, node.val);
                                        if (o != null) {
                                            node.val = o;
                                        }
                                        else {
                                            n = -1;
                                            ConcurrentHashMap.Node<K, V> next = node.next;
                                            if (node2 != null) {
                                                node2.next = next;
                                            }
                                            else {
                                                setTabAt(array, n3, next);
                                            }
                                        }
                                    }
                                    else if ((node instanceof ConcurrentHashMap.TreeBin)) {
                                        n2 = 1;
                                        ConcurrentHashMap.TreeBin treeBin = (ConcurrentHashMap.TreeBin)node;
                                        ConcurrentHashMap.TreeNode<Object, Object> root;
                                        ConcurrentHashMap.TreeNode<Object, Object> treeNode;
                                        if ((root = treeBin.root) != null) {
                                            treeNode = root.findTreeNode(spread, k, null);
                                        }
                                        else {
                                            treeNode = null;
                                        }
                                        o = biFunction.apply(k, (treeNode == null) ? null : treeNode.val);
                                        if (o != null) {
                                            if (treeNode != null) {
                                                treeNode.val = o;
                                            }
                                            else {
                                                n = 1;
                                                treeBin.putTreeVal(spread, k, o);
                                            }
                                        }
                                        else if (treeNode != null) {
                                            n = -1;
                                            if (treeBin.removeTreeNode(treeNode)) {
                                                setTabAt(array, n3, untreeify(treeBin.first));
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        if (n2 != 0) {
                            if (n2 >= 8) {
                                this.treeifyBin(array, n3);
                                break;
                            }
                            break;
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
        }
        if (n != 0) {
            this.addCount((long)n, n2);
            return o;
        }
        return o;
    }
    
    @Override
    public V merge(K k, V v, BiFunction<? super V, ? super V, ? extends V> biFunction) {
        if (k == null || v == null || biFunction == null) {
            throw new NullPointerException();
        }
        int spread = spread(k.hashCode());
        Object apply = null;
        int n = 0;
        int n2 = 0;
        ConcurrentHashMap.Node[] array = this.table;
        while (true) {
            int length;
            if (array == null || (length = array.length) == 0) {
                array = this.initTable();
            }
            else {
                int n3;
                ConcurrentHashMap.Node<K, V> node;
                if ((node = tabAt(array, n3 = (length - 1 & spread))) == null) {
                    if (casTabAt(array, n3, null, new ConcurrentHashMap.Node(spread, k, v))) {
                        n = 1;
                        apply = v;
                        break;
                    }
                    continue;
                }
                else {
                    int hash;
                    if ((hash = node.hash) == -1) {
                        array = this.helpTransfer(array, node);
                    }
                    else {
                        synchronized (node) {
                            Label_0469: {
                                if (tabAt(array, n3) == node) {
                                    if (hash >= 0) {
                                        n2 = 1;
                                        ConcurrentHashMap.Node<Object, Object> node2 = null;
                                        Object key;
                                        while (node.hash != spread || ((key = node.key) != k && (key == null || !k.equals(key)))) {
                                            node2 = node;
                                            if ((node = node.next) == null) {
                                                n = 1;
                                                apply = v;
                                                node.next = new ConcurrentHashMap.Node<Object, Object>(spread, k, apply);
                                                break Label_0469;
                                            }
                                            ++n2;
                                        }
                                        apply = biFunction.apply(node.val, v);
                                        if (apply != null) {
                                            node.val = apply;
                                        }
                                        else {
                                            n = -1;
                                            ConcurrentHashMap.Node<K, V> next = node.next;
                                            if (node2 != null) {
                                                node2.next = next;
                                            }
                                            else {
                                                setTabAt(array, n3, next);
                                            }
                                        }
                                    }
                                    else if ((node instanceof ConcurrentHashMap.TreeBin)) {
                                        n2 = 2;
                                        ConcurrentHashMap.TreeBin treeBin = (ConcurrentHashMap.TreeBin)node;
                                        ConcurrentHashMap.TreeNode<Object, Object> root = treeBin.root;
                                        ConcurrentHashMap.TreeNode<Object, Object> treeNode = (root == null) ? null : root.findTreeNode(spread, k, null);
                                        apply = ((treeNode == null) ? v : biFunction.apply(treeNode.val, v));
                                        if (apply != null) {
                                            if (treeNode != null) {
                                                treeNode.val = apply;
                                            }
                                            else {
                                                n = 1;
                                                treeBin.putTreeVal(spread, k, apply);
                                            }
                                        }
                                        else if (treeNode != null) {
                                            n = -1;
                                            if (treeBin.removeTreeNode(treeNode)) {
                                                setTabAt(array, n3, untreeify(treeBin.first));
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        if (n2 != 0) {
                            if (n2 >= 8) {
                                this.treeifyBin(array, n3);
                                break;
                            }
                            break;
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
        }
        if (n != 0) {
            this.addCount((long)n, n2);
            return apply;
        }
        return apply;
    }
    
    public boolean contains(Object o) {
        return this.containsValue(o);
    }
    
    public Enumeration<K> keys() {
        ConcurrentHashMap.Node[] table;
        int n = ((table = this.table) == null) ? 0 : table.length;
        return new ConcurrentHashMap.KeyIterator(table, n, 0, n, this);
    }
    
    public Enumeration<V> elements() {
        ConcurrentHashMap.Node[] table;
        int n = ((table = this.table) == null) ? 0 : table.length;
        return new ConcurrentHashMap.ValueIterator(table, n, 0, n, this);
    }
    
    public long mappingCount() {
        long sumCount = this.sumCount();
        return (sumCount < 0L) ? 0L : sumCount;
    }
    
    public static <K> ConcurrentHashMap.KeySetView<K, Boolean> newKeySet() {
        return new ConcurrentHashMap.KeySetView(new ConcurrentHashMap(), Boolean.TRUE);
    }
    
    public static <K> ConcurrentHashMap.KeySetView<K, Boolean> newKeySet(int n) {
        return new ConcurrentHashMap.KeySetView(new ConcurrentHashMap(n), Boolean.TRUE);
    }
    
    public ConcurrentHashMap.KeySetView<K, V> keySet(V v) {
        if (v == null) {
            throw new NullPointerException();
        }
        return new ConcurrentHashMap.KeySetView(this, v);
    }
    
    static final int resizeStamp(int i) {
        return Integer.numberOfLeadingZeros(i) | 1 << ConcurrentHashMap.RESIZE_STAMP_BITS - 1;
    }
    
    private final ConcurrentHashMap.Node<K, V>[] initTable() {
        ConcurrentHashMap.Node[] table;
        while (this.table == null || table.length == 0) {
            int sizeCtl;
            if ((sizeCtl = this.sizeCtl) < 0) {
                Thread.yield();
            }
            else {
                if (!ConcurrentHashMap.U.compareAndSwapInt(this, ConcurrentHashMap.SIZECTL, sizeCtl, -1)) {
                    continue;
                }
                try {
                    if (this.table == null || table.length == 0) {
                        int n = (sizeCtl > 0) ? sizeCtl : 16;
                        table = (ConcurrentHashMap.Node[])new ConcurrentHashMap.Node[n];
                        this.table = table;
                        sizeCtl = n - (n >>> 2);
                    }
                }
                finally {
                    this.sizeCtl = sizeCtl;
                }
            }
        }
        return table;
    }
    
    private final void addCount(long n, int n2) {
        long n3;
        Label_0120: {
            ConcurrentHashMap.CounterCell[] counterCells;
            if ((counterCells = this.counterCells) == null) {
                Unsafe u = ConcurrentHashMap.U;
                long basecount = ConcurrentHashMap.BASECOUNT;
                long baseCount = this.baseCount;
                if (u.compareAndSwapLong(this, basecount, baseCount, n3 = baseCount + n)) {
                    break Label_0120;
                }
            }
            boolean compareAndSwapLong = true;
            int n4;
            ConcurrentHashMap.CounterCell counterCell;
            if (counterCells != null && (n4 = counterCells.length - 1) >= 0 && (counterCell = counterCells[ThreadLocalRandom.getProbe() & n4]) != null) {
                Unsafe u2 = ConcurrentHashMap.U;
                ConcurrentHashMap.CounterCell o = counterCell;
                long cellvalue = ConcurrentHashMap.CELLVALUE;
                long value = counterCell.value;
                if ((compareAndSwapLong = u2.compareAndSwapLong(o, cellvalue, value, value + n)) != 0) {
                    if (n2 <= 1) {
                        return;
                    }
                    n3 = this.sumCount();
                    break Label_0120;
                }
            }
            this.fullAddCount(n, (boolean)compareAndSwapLong);
            return;
        }
        if (n2 >= 0) {
            int sizeCtl;
            ConcurrentHashMap.Node[] table;
            int length;
            while (n3 >= (long)(sizeCtl = this.sizeCtl) && (table = this.table) != null && (length = table.length) < 1073741824) {
                int resizeStamp = resizeStamp(length);
                if (sizeCtl < 0) {
                    ConcurrentHashMap.Node[] nextTable;
                    if (sizeCtl >>> ConcurrentHashMap.RESIZE_STAMP_SHIFT == resizeStamp && sizeCtl != resizeStamp + 1 && sizeCtl != resizeStamp + ConcurrentHashMap.MAX_RESIZERS && (nextTable = this.nextTable) != null) {
                        if (this.transferIndex <= 0) {
                            break;
                        }
                        if (ConcurrentHashMap.U.compareAndSwapInt(this, ConcurrentHashMap.SIZECTL, sizeCtl, sizeCtl + 1)) {
                            this.transfer(table, nextTable);
                        }
                    }
                    else {
                        break;
                    }
                }
                else if (ConcurrentHashMap.U.compareAndSwapInt(this, ConcurrentHashMap.SIZECTL, sizeCtl, (resizeStamp << ConcurrentHashMap.RESIZE_STAMP_SHIFT) + 2)) {
                    this.transfer(table, (ConcurrentHashMap$Node[])null);
                }
                n3 = this.sumCount();
            }
        }
    }
    
    final ConcurrentHashMap.Node<K, V>[] helpTransfer(ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.Node<K, V> node) {
        ConcurrentHashMap.Node[] nextTable;
        if (array != null && node instanceof ConcurrentHashMap.ForwardingNode && (nextTable = ((ConcurrentHashMap.ForwardingNode)node).nextTable) != null) {
            int resizeStamp = resizeStamp(array.length);
            int sizeCtl;
            while (nextTable == this.nextTable && this.table == array && (sizeCtl = this.sizeCtl) < 0 && sizeCtl >>> ConcurrentHashMap.RESIZE_STAMP_SHIFT == resizeStamp && sizeCtl != resizeStamp + 1 && sizeCtl != resizeStamp + ConcurrentHashMap.MAX_RESIZERS) {
                if (this.transferIndex <= 0) {
                    break;
                }
                if (!ConcurrentHashMap.U.compareAndSwapInt(this, ConcurrentHashMap.SIZECTL, sizeCtl, sizeCtl + 1)) {
                    continue;
                }
                this.transfer(array, nextTable);
                break;
            }
            return nextTable;
        }
        return this.table;
    }
    
    private final void tryPresize(int n) {
        int n2 = (n >= 536870912) ? 1073741824 : tableSizeFor(n + (n >>> 1) + 1);
        int sizeCtl;
        while ((sizeCtl = this.sizeCtl) >= 0) {
            ConcurrentHashMap.Node[] table = this.table;
            int length;
            if (table == null || (length = table.length) == 0) {
                n2 = ((sizeCtl > n2) ? sizeCtl : n2);
                if (ConcurrentHashMap.U.compareAndSwapInt(this, ConcurrentHashMap.SIZECTL, sizeCtl, -1)) {
                    try {
                        if (this.table == table) {
                            this.table = (ConcurrentHashMap.Node[])new ConcurrentHashMap.Node[n2];
                            sizeCtl = n2 - (n2 >>> 2);
                        }
                    }
                    finally {
                        this.sizeCtl = sizeCtl;
                    }
                }
                else {
                    continue;
                }
            }
            else if (n2 > sizeCtl) {
                if (length >= 1073741824) {
                    break;
                }
                if (table == this.table) {
                    int resizeStamp = resizeStamp(length);
                    if (sizeCtl < 0) {
                        ConcurrentHashMap.Node[] nextTable;
                        if (sizeCtl >>> ConcurrentHashMap.RESIZE_STAMP_SHIFT == resizeStamp && sizeCtl != resizeStamp + 1 && sizeCtl != resizeStamp + ConcurrentHashMap.MAX_RESIZERS && (nextTable = this.nextTable) != null) {
                            if (this.transferIndex <= 0) {
                                break;
                            }
                            if (ConcurrentHashMap.U.compareAndSwapInt(this, ConcurrentHashMap.SIZECTL, sizeCtl, sizeCtl + 1)) {
                                this.transfer(table, nextTable);
                            }
                            else {
                                continue;
                            }
                        }
                        else {
                            break;
                        }
                    }
                    else if (ConcurrentHashMap.U.compareAndSwapInt(this, ConcurrentHashMap.SIZECTL, sizeCtl, (resizeStamp << ConcurrentHashMap.RESIZE_STAMP_SHIFT) + 2)) {
                        this.transfer(table, (ConcurrentHashMap$Node[])null);
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
            else {
                break;
            }
        }
    }
    
    private final void transfer(ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.Node<K, V>[] array2) {
        int length = array.length;
        int n;
        if ((n = ((ConcurrentHashMap.NCPU > 1) ? ((length >>> 3) / ConcurrentHashMap.NCPU) : length)) < 16) {
            n = 16;
        }
        if (array2 == null) {
            try {
                array2 = (ConcurrentHashMap.Node[])new ConcurrentHashMap.Node[length << 1];
            }
            catch (Throwable t) {
                this.sizeCtl = Integer.MAX_VALUE;
                return;
            }
            this.nextTable = array2;
            this.transferIndex = length;
        }
        int length2 = array2.length;
        ConcurrentHashMap.ForwardingNode o = new ConcurrentHashMap.ForwardingNode(array2);
        int casTab = 1;
        int n2 = 0;
        int n3 = 0;
        int n4 = 0;
        while (true) {
            if (casTab != 0) {
                if (--n3 >= n4 || n2 != 0) {
                    casTab = 0;
                }
                else {
                    int transferIndex;
                    if ((transferIndex = this.transferIndex) <= 0) {
                        n3 = -1;
                        casTab = 0;
                    }
                    else {
                        int expected;
                        int n9;
                        if (ConcurrentHashMap.U.compareAndSwapInt(this, ConcurrentHashMap.TRANSFERINDEX, expected, n9 = (((expected = transferIndex) > n) ? (transferIndex - n) : 0))) {
                            n4 = n9;
                            n3 = transferIndex - 1;
                            casTab = 0;
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
            else if (n3 < 0 || n3 >= length || n3 + length >= length2) {
                if (n2 != 0) {
                    this.nextTable = null;
                    this.table = array2;
                    this.sizeCtl = (length << 1) - (length >>> 1);
                    return;
                }
                int sizeCtl;
                if (ConcurrentHashMap.U.compareAndSwapInt(this, ConcurrentHashMap.SIZECTL, sizeCtl = this.sizeCtl, sizeCtl - 1)) {
                    if (sizeCtl - 2 != resizeStamp(length) << ConcurrentHashMap.RESIZE_STAMP_SHIFT) {
                        return;
                    }
                    casTab = (n2 = 1);
                    n3 = length;
                }
                else {
                    continue;
                }
            }
            else {
                ConcurrentHashMap.Node node;
                if ((node = tabAt(array, n3)) == null) {
                    casTab = casTabAt(array, n3, null, o);
                }
                else {
                    int hash;
                    if ((hash = node.hash) == -1) {
                        casTab = 1;
                    }
                    else {
                        synchronized (node) {
                            if (tabAt(array, n3) == node) {
                                if (hash >= 0) {
                                    int n7 = hash & length;
                                    for (ConcurrentHashMap.Node node2 = node.next; node2 != null; node2 = node2.next) {
                                        int n8 = node2.hash & length;
                                        if (n8 != n7) {
                                            n7 = n8;
                                            node = node2;
                                        }
                                    }
                                    if (n7 == 0) {
                                    }
                                    else {
                                        node = null;
                                    }
                                    for (; node != node; node = node.next) {
                                        int hash3 = node.hash;
                                        Object key = node.key;
                                        Object val = node.val;
                                        if ((hash3 & length) == 0x0) {
                                            node = new ConcurrentHashMap.Node(hash3, key, val, node);
                                        }
                                        else {
                                            ConcurrentHashMap.Node o7 = new ConcurrentHashMap.Node(hash3, key, val, node);
                                        }
                                    }
                                    setTabAt(array2, n3, node);
                                    setTabAt(array2, n3 + length, node);
                                    setTabAt(array, n3, o);
                                    casTab = 1;
                                }
                                else if ((node instanceof ConcurrentHashMap.TreeBin)) {
                                    ConcurrentHashMap.TreeBin<Object, Object> treeBin = (ConcurrentHashMap.TreeBin)node;
                                    Object o2 = null;
                                    Object prev = null;
                                    Object o3 = null;
                                    Object prev2 = null;
                                    int n5 = 0;
                                    int n6 = 0;
                                    for (ConcurrentHashMap.TreeNode treeNode = treeBin.first; treeNode != null; treeNode = treeNode.next) {
                                        int hash2 = treeNode.hash;
                                        ConcurrentHashMap.TreeNode o4 = new ConcurrentHashMap.TreeNode(hash2, treeNode.key, treeNode.val, null);
                                        if ((hash2 & length) == 0x0) {
                                            if ((o4.prev = prev) == null) {
                                                o2 = o4;
                                            }
                                            else {
                                                prev.next = o4;
                                            }
                                            prev = o4;
                                            ++n5;
                                        }
                                        else {
                                            if ((o4.prev = prev2) == null) {
                                                o3 = o4;
                                            }
                                            else {
                                                prev2.next = o4;
                                            }
                                            prev2 = o4;
                                            ++n6;
                                        }
                                    }
                                    Object o5 = (n5 <= 6) ? untreeify(o2) : ((n6 != 0) ? new ConcurrentHashMap.TreeBin(o2) : treeBin);
                                    Object o6 = (n6 <= 6) ? untreeify(o3) : ((n5 != 0) ? new ConcurrentHashMap.TreeBin(o3) : treeBin);
                                    setTabAt(array2, n3, o5);
                                    setTabAt(array2, n3 + length, o6);
                                    setTabAt(array, n3, o);
                                    casTab = 1;
                                }
                                else {
                                    continue;
                                }
                            }
                            else {
                                continue;
                            }
                        }
                    }
                }
            }
        }
    }
    
    final long sumCount() {
        ConcurrentHashMap.CounterCell[] counterCells = this.counterCells;
        long baseCount = this.baseCount;
        if (counterCells != null) {
            for (int i = 0; i < counterCells.length; ++i) {
                ConcurrentHashMap.CounterCell counterCell;
                if ((counterCell = counterCells[i]) != null) {
                    baseCount += counterCell.value;
                }
            }
        }
        return baseCount;
    }
    
    private final void fullAddCount(long n, boolean b) {
        int n2;
        if ((n2 = ThreadLocalRandom.getProbe()) == 0) {
            ThreadLocalRandom.localInit();
            n2 = ThreadLocalRandom.getProbe();
            b = true;
        }
        int n3 = 0;
        while (true) {
            ConcurrentHashMap.CounterCell[] counterCells;
            int length;
            if ((counterCells = this.counterCells) != null && (length = counterCells.length) > 0) {
                ConcurrentHashMap.CounterCell counterCell;
                if ((counterCell = counterCells[length - 1 & n2]) == null) {
                    if (this.cellsBusy == 0) {
                        ConcurrentHashMap.CounterCell o2 = new CounterCell(n);
                        if (this.cellsBusy == 0 && ConcurrentHashMap.U.compareAndSwapInt(this, ConcurrentHashMap.CELLSBUSY, 0, 1)) {
                            boolean n5 = false;
                            try {
                                ConcurrentHashMap.CounterCell[] counterCells4;
                                int length2;
                                int n6;
                                if ((counterCells4 = this.counterCells) != null && (length2 = counterCells4.length) > 0 && counterCells4[n6 = (length2 - 1 & n2)] == null) {
                                    counterCells4[n6] = o2;
                                    n5 = true;
                                }
                            }
                            finally {
                                this.cellsBusy = 0;
                            }
                            if (n5 == 0) {
                                continue;
                            }
                            break;
                        }
                    }
                    n3 = 0;
                }
                else if (!b) {
                    b = true;
                }
                else {
                    Unsafe u2 = ConcurrentHashMap.U;
                    ConcurrentHashMap.CounterCell o = counterCell;
                    long cellvalue = ConcurrentHashMap.CELLVALUE;
                    long value = counterCell.value;
                    if (u2.compareAndSwapLong(o, cellvalue, value, value + n)) {
                        break;
                    }
                    if (this.counterCells != counterCells || length >= ConcurrentHashMap.NCPU) {
                        n3 = 0;
                    }
                    else if (n3 == 0) {
                        n3 = 1;
                    }
                    else if (this.cellsBusy == 0 && ConcurrentHashMap.U.compareAndSwapInt(this, ConcurrentHashMap.CELLSBUSY, 0, 1)) {
                        try {
                            if (this.counterCells == counterCells) {
                                ConcurrentHashMap.CounterCell[] counterCells3 = new ConcurrentHashMap.CounterCell[length << 1];
                                for (int i = 0; i < length; ++i) {
                                    counterCells3[i] = counterCells[i];
                                }
                                this.counterCells = counterCells3;
                            }
                        }
                        finally {
                            this.cellsBusy = 0;
                        }
                        n3 = 0;
                        continue;
                    }
                }
                n2 = ThreadLocalRandom.advanceProbe(n2);
            }
            else if (this.cellsBusy == 0 && this.counterCells == counterCells && ConcurrentHashMap.U.compareAndSwapInt(this, ConcurrentHashMap.CELLSBUSY, 0, 1)) {
                boolean n4 = false;
                try {
                    if (this.counterCells == counterCells) {
                        ConcurrentHashMap.CounterCell[] counterCells2 = new ConcurrentHashMap.CounterCell[2];
                        counterCells2[n2 & 0x1] = new CounterCell(n);
                        this.counterCells = counterCells2;
                        n4 = true;
                    }
                }
                finally {
                    this.cellsBusy = 0;
                }
                if (!(n4 == 0)) {
                    break;
                }
                continue;
            }
            else {
                Unsafe u = ConcurrentHashMap.U;
                long basecount = ConcurrentHashMap.BASECOUNT;
                long baseCount = this.baseCount;
                if (u.compareAndSwapLong(this, basecount, baseCount, baseCount + n)) {
                    break;
                }
                continue;
            }
        }
    }
    
    private final void treeifyBin(ConcurrentHashMap.Node<K, V>[] array, int n) {
        if (array != null) {
            int length;
            if ((length = array.length) < 64) {
                this.tryPresize(length << 1);
            }
            else {
                ConcurrentHashMap.Node<K, V> node;
                if ((node = tabAt(array, n)) != null && node.hash >= 0) {
                    synchronized (node) {
                        if (tabAt(array, n) == node) {
                            Object o = null;
                            Object prev = null;
                            Object next;
                            for (; node != null; prev = next, node = node.next) {
                                next = new ConcurrentHashMap.TreeNode(node.hash, node.key, node.val);
                                if ((next.prev = prev) == null) {
                                    o = next;
                                }
                                else {
                                    prev.next = next;
                                }
                            }
                            setTabAt(array, n, new ConcurrentHashMap.TreeBin(o));
                        }
                    }
                }
            }
        }
    }
    
    static <K, V> ConcurrentHashMap.Node<K, V> untreeify(ConcurrentHashMap.Node<K, V> next) {
        Object o = null;
        Object o2 = null;
        Object next2;
        for (; next != null; o2 = next2, next = next.next) {
            next2 = new ConcurrentHashMap.Node(next.hash, next.key, next.val);
            if (o2 == null) {
                o = next2;
            }
            else {
                o2.next = next2;
            }
        }
        return o;
    }
    
    final int batchFor(long n) {
        long sumCount;
        if (n == Long.MAX_VALUE || (sumCount = this.sumCount()) <= 1L || sumCount < n) {
            return 0;
        }
        int n2 = ForkJoinPool.getCommonPoolParallelism() << 2;
        long n3;
        return (n <= 0L || (n3 = sumCount / n) >= (long)n2) ? n2 : ((int)n3);
    }
    
    public void forEach(long n, BiConsumer<? super K, ? super V> biConsumer) {
        if (biConsumer == null) {
            throw new NullPointerException();
        }
        new ConcurrentHashMap.ForEachMappingTask(null, this.batchFor(n), 0, 0, this.table, biConsumer).invoke();
        return;
    }
    
    public <U> void forEach(long n, BiFunction<? super K, ? super V, ? extends U> biFunction, Consumer<? super U> consumer) {
        if (biFunction == null || consumer == null) {
            throw new NullPointerException();
        }
        new ConcurrentHashMap.ForEachTransformedMappingTask(null, this.batchFor(n), 0, 0, this.table, biFunction, consumer).invoke();
        return;
    }
    
    public <U> U search(long n, BiFunction<? super K, ? super V, ? extends U> biFunction) {
        if (biFunction == null) {
            throw new NullPointerException();
        }
        return new ConcurrentHashMap.SearchMappingsTask(null, this.batchFor(n), 0, 0, this.table, biFunction, new AtomicReference()).invoke();
    }
    
    public <U> U reduce(long n, BiFunction<? super K, ? super V, ? extends U> biFunction, BiFunction<? super U, ? super U, ? extends U> biFunction2) {
        if (biFunction == null || biFunction2 == null) {
            throw new NullPointerException();
        }
        return new ConcurrentHashMap.MapReduceMappingsTask(null, this.batchFor(n), 0, 0, this.table, null, biFunction, biFunction2).invoke();
    }
    
    public double reduceToDouble(long n, ToDoubleBiFunction<? super K, ? super V> toDoubleBiFunction, double n2, DoubleBinaryOperator doubleBinaryOperator) {
        if (toDoubleBiFunction == null || doubleBinaryOperator == null) {
            throw new NullPointerException();
        }
        return (double)new ConcurrentHashMap.MapReduceMappingsToDoubleTask(null, this.batchFor(n), 0, 0, this.table, null, toDoubleBiFunction, n2, doubleBinaryOperator).invoke();
    }
    
    public long reduceToLong(long n, ToLongBiFunction<? super K, ? super V> toLongBiFunction, long n2, LongBinaryOperator longBinaryOperator) {
        if (toLongBiFunction == null || longBinaryOperator == null) {
            throw new NullPointerException();
        }
        return (long)new ConcurrentHashMap.MapReduceMappingsToLongTask(null, this.batchFor(n), 0, 0, this.table, null, toLongBiFunction, n2, longBinaryOperator).invoke();
    }
    
    public int reduceToInt(long n, ToIntBiFunction<? super K, ? super V> toIntBiFunction, int n2, IntBinaryOperator intBinaryOperator) {
        if (toIntBiFunction == null || intBinaryOperator == null) {
            throw new NullPointerException();
        }
        return (int)new ConcurrentHashMap.MapReduceMappingsToIntTask(null, this.batchFor(n), 0, 0, this.table, null, toIntBiFunction, n2, intBinaryOperator).invoke();
    }
    
    public void forEachKey(long n, Consumer<? super K> consumer) {
        if (consumer == null) {
            throw new NullPointerException();
        }
        new ConcurrentHashMap.ForEachKeyTask(null, this.batchFor(n), 0, 0, this.table, consumer).invoke();
        return;
    }
    
    public <U> void forEachKey(long n, Function<? super K, ? extends U> function, Consumer<? super U> consumer) {
        if (function == null || consumer == null) {
            throw new NullPointerException();
        }
        new ConcurrentHashMap.ForEachTransformedKeyTask(null, this.batchFor(n), 0, 0, this.table, function, consumer).invoke();
        return;
    }
    
    public <U> U searchKeys(long n, Function<? super K, ? extends U> function) {
        if (function == null) {
            throw new NullPointerException();
        }
        return new ConcurrentHashMap.SearchKeysTask(null, this.batchFor(n), 0, 0, this.table, function, new AtomicReference()).invoke();
    }
    
    public K reduceKeys(long n, BiFunction<? super K, ? super K, ? extends K> biFunction) {
        if (biFunction == null) {
            throw new NullPointerException();
        }
        return new ConcurrentHashMap.ReduceKeysTask(null, this.batchFor(n), 0, 0, this.table, null, biFunction).invoke();
    }
    
    public <U> U reduceKeys(long n, Function<? super K, ? extends U> function, BiFunction<? super U, ? super U, ? extends U> biFunction) {
        if (function == null || biFunction == null) {
            throw new NullPointerException();
        }
        return new ConcurrentHashMap.MapReduceKeysTask(null, this.batchFor(n), 0, 0, this.table, null, function, biFunction).invoke();
    }
    
    public double reduceKeysToDouble(long n, ToDoubleFunction<? super K> toDoubleFunction, double n2, DoubleBinaryOperator doubleBinaryOperator) {
        if (toDoubleFunction == null || doubleBinaryOperator == null) {
            throw new NullPointerException();
        }
        return (double)new ConcurrentHashMap.MapReduceKeysToDoubleTask(null, this.batchFor(n), 0, 0, this.table, null, toDoubleFunction, n2, doubleBinaryOperator).invoke();
    }
    
    public long reduceKeysToLong(long n, ToLongFunction<? super K> toLongFunction, long n2, LongBinaryOperator longBinaryOperator) {
        if (toLongFunction == null || longBinaryOperator == null) {
            throw new NullPointerException();
        }
        return (long)new ConcurrentHashMap.MapReduceKeysToLongTask(null, this.batchFor(n), 0, 0, this.table, null, toLongFunction, n2, longBinaryOperator).invoke();
    }
    
    public int reduceKeysToInt(long n, ToIntFunction<? super K> toIntFunction, int n2, IntBinaryOperator intBinaryOperator) {
        if (toIntFunction == null || intBinaryOperator == null) {
            throw new NullPointerException();
        }
        return (int)new ConcurrentHashMap.MapReduceKeysToIntTask(null, this.batchFor(n), 0, 0, this.table, null, toIntFunction, n2, intBinaryOperator).invoke();
    }
    
    public void forEachValue(long n, Consumer<? super V> consumer) {
        if (consumer == null) {
            throw new NullPointerException();
        }
        new ConcurrentHashMap.ForEachValueTask(null, this.batchFor(n), 0, 0, this.table, consumer).invoke();
        return;
    }
    
    public <U> void forEachValue(long n, Function<? super V, ? extends U> function, Consumer<? super U> consumer) {
        if (function == null || consumer == null) {
            throw new NullPointerException();
        }
        new ConcurrentHashMap.ForEachTransformedValueTask(null, this.batchFor(n), 0, 0, this.table, function, consumer).invoke();
        return;
    }
    
    public <U> U searchValues(long n, Function<? super V, ? extends U> function) {
        if (function == null) {
            throw new NullPointerException();
        }
        return new ConcurrentHashMap.SearchValuesTask(null, this.batchFor(n), 0, 0, this.table, function, new AtomicReference()).invoke();
    }
    
    public V reduceValues(long n, BiFunction<? super V, ? super V, ? extends V> biFunction) {
        if (biFunction == null) {
            throw new NullPointerException();
        }
        return new ConcurrentHashMap.ReduceValuesTask(null, this.batchFor(n), 0, 0, this.table, null, biFunction).invoke();
    }
    
    public <U> U reduceValues(long n, Function<? super V, ? extends U> function, BiFunction<? super U, ? super U, ? extends U> biFunction) {
        if (function == null || biFunction == null) {
            throw new NullPointerException();
        }
        return new ConcurrentHashMap.MapReduceValuesTask(null, this.batchFor(n), 0, 0, this.table, null, function, biFunction).invoke();
    }
    
    public double reduceValuesToDouble(long n, ToDoubleFunction<? super V> toDoubleFunction, double n2, DoubleBinaryOperator doubleBinaryOperator) {
        if (toDoubleFunction == null || doubleBinaryOperator == null) {
            throw new NullPointerException();
        }
        return (double)new ConcurrentHashMap.MapReduceValuesToDoubleTask(null, this.batchFor(n), 0, 0, this.table, null, toDoubleFunction, n2, doubleBinaryOperator).invoke();
    }
    
    public long reduceValuesToLong(long n, ToLongFunction<? super V> toLongFunction, long n2, LongBinaryOperator longBinaryOperator) {
        if (toLongFunction == null || longBinaryOperator == null) {
            throw new NullPointerException();
        }
        return (long)new ConcurrentHashMap.MapReduceValuesToLongTask(null, this.batchFor(n), 0, 0, this.table, null, toLongFunction, n2, longBinaryOperator).invoke();
    }
    
    public int reduceValuesToInt(long n, ToIntFunction<? super V> toIntFunction, int n2, IntBinaryOperator intBinaryOperator) {
        if (toIntFunction == null || intBinaryOperator == null) {
            throw new NullPointerException();
        }
        return (int)new ConcurrentHashMap.MapReduceValuesToIntTask(null, this.batchFor(n), 0, 0, this.table, null, toIntFunction, n2, intBinaryOperator).invoke();
    }
    
    public void forEachEntry(long n, Consumer<? super Map.Entry<K, V>> consumer) {
        if (consumer == null) {
            throw new NullPointerException();
        }
        new ConcurrentHashMap.ForEachEntryTask(null, this.batchFor(n), 0, 0, this.table, consumer).invoke();
        return;
    }
    
    public <U> void forEachEntry(long n, Function<Map.Entry<K, V>, ? extends U> function, Consumer<? super U> consumer) {
        if (function == null || consumer == null) {
            throw new NullPointerException();
        }
        new ConcurrentHashMap.ForEachTransformedEntryTask(null, this.batchFor(n), 0, 0, this.table, function, consumer).invoke();
        return;
    }
    
    public <U> U searchEntries(long n, Function<Map.Entry<K, V>, ? extends U> function) {
        if (function == null) {
            throw new NullPointerException();
        }
        return new ConcurrentHashMap.SearchEntriesTask(null, this.batchFor(n), 0, 0, this.table, function, new AtomicReference()).invoke();
    }
    
    public Map.Entry<K, V> reduceEntries(long n, BiFunction<Map.Entry<K, V>, Map.Entry<K, V>, ? extends Map.Entry<K, V>> biFunction) {
        if (biFunction == null) {
            throw new NullPointerException();
        }
        return (Map.Entry)new ConcurrentHashMap.ReduceEntriesTask(null, this.batchFor(n), 0, 0, this.table, null, biFunction).invoke();
    }
    
    public <U> U reduceEntries(long n, Function<Map.Entry<K, V>, ? extends U> function, BiFunction<? super U, ? super U, ? extends U> biFunction) {
        if (function == null || biFunction == null) {
            throw new NullPointerException();
        }
        return new ConcurrentHashMap.MapReduceEntriesTask(null, this.batchFor(n), 0, 0, this.table, null, function, biFunction).invoke();
    }
    
    public double reduceEntriesToDouble(long n, ToDoubleFunction<Map.Entry<K, V>> toDoubleFunction, double n2, DoubleBinaryOperator doubleBinaryOperator) {
        if (toDoubleFunction == null || doubleBinaryOperator == null) {
            throw new NullPointerException();
        }
        return (double)new ConcurrentHashMap.MapReduceEntriesToDoubleTask(null, this.batchFor(n), 0, 0, this.table, null, toDoubleFunction, n2, doubleBinaryOperator).invoke();
    }
    
    public long reduceEntriesToLong(long n, ToLongFunction<Map.Entry<K, V>> toLongFunction, long n2, LongBinaryOperator longBinaryOperator) {
        if (toLongFunction == null || longBinaryOperator == null) {
            throw new NullPointerException();
        }
        return (long)new ConcurrentHashMap.MapReduceEntriesToLongTask(null, this.batchFor(n), 0, 0, this.table, null, toLongFunction, n2, longBinaryOperator).invoke();
    }
    
    public int reduceEntriesToInt(long n, ToIntFunction<Map.Entry<K, V>> toIntFunction, int n2, IntBinaryOperator intBinaryOperator) {
        if (toIntFunction == null || intBinaryOperator == null) {
            throw new NullPointerException();
        }
        return (int)new ConcurrentHashMap.MapReduceEntriesToIntTask(null, this.batchFor(n), 0, 0, this.table, null, toIntFunction, n2, intBinaryOperator).invoke();
    }
    
    static {
        ConcurrentHashMap.RESIZE_STAMP_BITS = 16;
        MAX_RESIZERS = (1 << 32 - ConcurrentHashMap.RESIZE_STAMP_BITS) - 1;
        RESIZE_STAMP_SHIFT = 32 - ConcurrentHashMap.RESIZE_STAMP_BITS;
        NCPU = Runtime.getRuntime().availableProcessors();
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("segments", ConcurrentHashMap.Segment[].class), new ObjectStreamField("segmentMask", Integer.TYPE), new ObjectStreamField("segmentShift", Integer.TYPE) };
        try {
            U = Unsafe.getUnsafe();
            Class<Object> clazz = ConcurrentHashMap.class;
            SIZECTL = ConcurrentHashMap.U.objectFieldOffset(clazz.getDeclaredField("sizeCtl"));
            TRANSFERINDEX = ConcurrentHashMap.U.objectFieldOffset(clazz.getDeclaredField("transferIndex"));
            BASECOUNT = ConcurrentHashMap.U.objectFieldOffset(clazz.getDeclaredField("baseCount"));
            CELLSBUSY = ConcurrentHashMap.U.objectFieldOffset(clazz.getDeclaredField("cellsBusy"));
            CELLVALUE = ConcurrentHashMap.U.objectFieldOffset(ConcurrentHashMap.CounterCell.class.getDeclaredField("value"));
            Class<Object> clazz2 = ConcurrentHashMap.Node[].class;
            ABASE = (long)ConcurrentHashMap.U.arrayBaseOffset(clazz2);
            int arrayIndexScale = ConcurrentHashMap.U.arrayIndexScale(clazz2);
            if ((arrayIndexScale & arrayIndexScale - 1) != 0x0) {
                throw new Error("data type scale not a power of two");
            }
            ASHIFT = 31 - Integer.numberOfLeadingZeros(arrayIndexScale);
        }
        catch (Exception cause) {
            throw new Error(cause);
        }
    }
    
    static class Node<K, V> implements Map.Entry<K, V>
    {
        final int hash;
        final K key;
        volatile V val;
        volatile ConcurrentHashMap.Node<K, V> next;
        
        Node(int hash, K key, V val, ConcurrentHashMap.Node<K, V> next) {
            this.hash = hash;
            this.key = key;
            this.val = val;
            this.next = next;
        }
        
        @Override
        public final K getKey() {
            return this.key;
        }
        
        @Override
        public final V getValue() {
            return this.val;
        }
        
        @Override
        public final int hashCode() {
            return this.key.hashCode() ^ this.val.hashCode();
        }
        
        @Override
        public final String toString() {
            return this.key + "=" + this.val;
        }
        
        @Override
        public final V setValue(V v) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public final boolean equals(Object o) {
            Map.Entry<Object, Object> entry;
            Object key;
            Object value;
            Object val;
            return o instanceof Map.Entry && (key = (entry = (Map.Entry)o).getKey()) != null && (value = entry.getValue()) != null && (key == this.key || key.equals(this.key)) && (value == (val = this.val) || value.equals(val));
        }
        
        ConcurrentHashMap.Node<K, V> find(int n, Object o) {
            if (o != null) {
                Object key;
                while (this.hash != n || ((key = this.key) != o && (key == null || !o.equals(key)))) {
                    if ((this = this.next) == null) {
                        return null;
                    }
                    continue;
                }
                return this;
            }
            return null;
        }
    }
    
    static class Segment<K, V> extends ReentrantLock implements Serializable
    {
        private static final long serialVersionUID = 2249069246763182397L;
        final float loadFactor;
        
        Segment(float loadFactor) {
            this.loadFactor = loadFactor;
        }
    }
    
    static final class ForwardingNode<K, V> extends ConcurrentHashMap.Node<K, V>
    {
        final ConcurrentHashMap.Node<K, V>[] nextTable;
        
        ForwardingNode(ConcurrentHashMap.Node<K, V>[] nextTable) {
            super(-1, null, null, null);
            this.nextTable = nextTable;
        }
        
        @Override
        ConcurrentHashMap.Node<K, V> find(int n, Object o) {
            ConcurrentHashMap.Node[] array = this.nextTable;
            int length;
            ConcurrentHashMap.Node<K, V> node;
        Label_0005:
            while (o != null && array != null && (length = array.length) != 0 && (node = ConcurrentHashMap.tabAt(array, length - 1 & n)) != null) {
                int hash;
                Object key;
                while ((hash = node.hash) != n || ((key = node.key) != o && (key == null || !o.equals(key)))) {
                    if (hash < 0) {
                        if ((node instanceof ConcurrentHashMap.ForwardingNode)) {
                            array = ((ConcurrentHashMap.ForwardingNode)node).nextTable;
                            continue Label_0005;
                        }
                        return node.find(n, o);
                    }
                    else {
                        if ((node = node.next) == null) {
                            return null;
                        }
                        continue;
                    }
                }
                return node;
            }
            return null;
        }
    }
    
    static final class ReservationNode<K, V> extends ConcurrentHashMap.Node<K, V>
    {
        ReservationNode() {
            super(-3, null, null, null);
        }
        
        @Override
        ConcurrentHashMap.Node<K, V> find(int n, Object o) {
            return null;
        }
    }
    
    @Contended
    static final class CounterCell
    {
        volatile long value;
        
        CounterCell(long value) {
            this.value = value;
        }
    }
    
    static final class TreeNode<K, V> extends ConcurrentHashMap.Node<K, V>
    {
        ConcurrentHashMap.TreeNode<K, V> parent;
        ConcurrentHashMap.TreeNode<K, V> left;
        ConcurrentHashMap.TreeNode<K, V> right;
        ConcurrentHashMap.TreeNode<K, V> prev;
        boolean red;
        
        TreeNode(int n, K k, V v, ConcurrentHashMap.Node<K, V> node, ConcurrentHashMap.TreeNode<K, V> parent) {
            super(n, k, v, node);
            this.parent = parent;
        }
        
        @Override
        ConcurrentHashMap.Node<K, V> find(int n, Object o) {
            return this.findTreeNode(n, o, null);
        }
        
        final ConcurrentHashMap.TreeNode<K, V> findTreeNode(int p0, Object p1, Class<?> p2) {
            // Bytecode:
            // 
            //     1: ifnull          175
            //     4: aload_0        
            //     5: astore          4
            //     7: aload           4
            //     9: getfield        java/util/concurrent/ConcurrentHashMap$TreeNode.left:Ljava/util/concurrent/ConcurrentHashMap$TreeNode;
            //    12: astore          9
            //    14: aload           4
            //    16: getfield        java/util/concurrent/ConcurrentHashMap$TreeNode.right:Ljava/util/concurrent/ConcurrentHashMap$TreeNode;
            //    19: astore          10
            //    21: aload           4
            //    23: getfield        java/util/concurrent/ConcurrentHashMap$TreeNode.hash:I
            //    26: dup            
            //    27: istore          5
            //    29: iload_1        
            //    30: if_icmple       40
            //    33: aload           9
            //    35: astore          4
            //    37: goto            170
            //    40: iload           5
            //    42: iload_1        
            //    43: if_icmpge       53
            //    46: aload           10
            //    48: astore          4
            //    50: goto            170
            //    53: aload           4
            //    55: getfield        java/util/concurrent/ConcurrentHashMap$TreeNode.key:Ljava/lang/Object;
            //    58: dup            
            //    59: astore          7
            //    61: aload_2        
            //    62: if_acmpeq       79
            //    65: aload           7
            //    67: ifnull          82
            //    70: aload_2        
            //    71: aload           7
            //    73: invokevirtual   java/lang/Object.equals:(Ljava/lang/Object;)Z
            //    76: ifeq            82
            //    79: aload           4
            //    81: areturn        
            //    82: aload           9
            //    84: ifnonnull       94
            //    87: aload           10
            //    89: astore          4
            //    91: goto            170
            //    94: aload           10
            //    96: ifnonnull       106
            //    99: aload           9
            //   101: astore          4
            //   103: goto            170
            //   106: aload_3        
            //   107: ifnonnull       119
            //   110: aload_2        
            //   111: invokestatic    java/util/concurrent/ConcurrentHashMap.comparableClassFor:(Ljava/lang/Object;)Ljava/lang/Class;
            //   114: dup            
            //   115: astore_3       
            //   116: ifnull          149
            //   119: aload_3        
            //   120: aload_2        
            //   121: aload           7
            //   123: invokestatic    java/util/concurrent/ConcurrentHashMap.compareComparables:(Ljava/lang/Class;Ljava/lang/Object;Ljava/lang/Object;)I
            //   126: dup            
            //   127: istore          6
            //   129: ifeq            149
            //   132: iload           6
            //   134: ifge            142
            //   137: aload           9
            //   139: goto            144
            //   142: aload           10
            //   144: astore          4
            //   146: goto            170
            //   149: aload           10
            //   151: iload_1        
            //   152: aload_2        
            //   153: aload_3        
            //   154: invokevirtual   java/util/concurrent/ConcurrentHashMap$TreeNode.findTreeNode:(ILjava/lang/Object;Ljava/lang/Class;)Ljava/util/concurrent/ConcurrentHashMap$TreeNode;
            //   157: dup            
            //   158: astore          8
            //   160: ifnull          166
            //   163: aload           8
            //   165: areturn        
            //   166: aload           9
            //   168: astore          4
            //   170: aload           4
            //   172: ifnonnull       7
            //   175: aconst_null    
            //   176: areturn        
            //    Signature:
            //  (ILjava/lang/Object;Ljava/lang/Class<*>;)Ljava/util/concurrent/ConcurrentHashMap$TreeNode<TK;TV;>;
            //    StackMapTable: 00 0E FC 00 07 07 00 22 FF 00 20 00 0B 07 00 22 01 07 00 1F 07 00 1E 07 00 22 01 00 00 00 07 00 22 07 00 22 00 00 0C FF 00 19 00 0B 07 00 22 01 07 00 1F 07 00 1E 07 00 22 01 00 07 00 1F 00 07 00 22 07 00 22 00 00 02 0B 0B 0C FF 00 16 00 0B 07 00 22 01 07 00 1F 07 00 1E 07 00 22 01 01 07 00 1F 00 07 00 22 07 00 22 00 00 41 07 00 22 FF 00 04 00 0B 07 00 22 01 07 00 1F 07 00 1E 07 00 22 01 00 07 00 1F 00 07 00 22 07 00 22 00 00 FF 00 10 00 0B 07 00 22 01 07 00 1F 07 00 1E 07 00 22 01 00 07 00 1F 07 00 22 07 00 22 07 00 22 00 00 FF 00 03 00 05 07 00 22 01 07 00 1F 07 00 1E 07 00 22 00 00 FA 00 04
            // 
            throw new IllegalStateException("An error occurred while decompiling this method.");
        }
    }
    
    static final class TreeBin<K, V> extends ConcurrentHashMap.Node<K, V>
    {
        ConcurrentHashMap.TreeNode<K, V> root;
        volatile ConcurrentHashMap.TreeNode<K, V> first;
        volatile Thread waiter;
        volatile int lockState;
        static final int WRITER = 1;
        static final int WAITER = 2;
        static final int READER = 4;
        private static final Unsafe U;
        private static final long LOCKSTATE;
        
        static int tieBreakOrder(Object o, Object o2) {
            int compareTo;
            if (o == null || o2 == null || (compareTo = o.getClass().getName().compareTo(o2.getClass().getName())) == 0) {
                compareTo = ((System.identityHashCode(o) <= System.identityHashCode(o2)) ? -1 : 1);
            }
            return compareTo;
        }
        
        TreeBin(ConcurrentHashMap.TreeNode<K, V> left) {
            super(-2, null, null, null);
            this.first = left;
            ConcurrentHashMap.TreeNode balanceInsertion = null;
            ConcurrentHashMap.TreeNode<Object, Object> treeNode;
            ConcurrentHashMap.TreeNode<Object, Object> treeNode2;
            ConcurrentHashMap.TreeNode<Object, Object> treeNode3;
            ConcurrentHashMap.TreeNode<K, V> treeNode4;
            Object key;
            int hash;
            Class<Object> comparableClass;
            Object key2;
            int hash2;
            int n;
            for (; left != null; left = treeNode) {
                treeNode = (ConcurrentHashMap.TreeNode)left.next;
                treeNode2 = left;
                treeNode3 = left;
                treeNode4 = null;
                treeNode3.right = treeNode4;
                treeNode2.left = treeNode4;
                if (balanceInsertion == null) {
                    left.parent = null;
                    left.red = false;
                    balanceInsertion = left;
                }
                else {
                    key = left.key;
                    hash = left.hash;
                    comparableClass = null;
                    while (true) {
                        key2 = balanceInsertion.key;
                        if ((hash2 = balanceInsertion.hash) > hash) {
                            n = -1;
                        }
                        else if (hash2 < hash) {
                            n = 1;
                        }
                        else if ((comparableClass == null && (comparableClass = ConcurrentHashMap.comparableClassFor(key)) == null) || (n = ConcurrentHashMap.compareComparables(comparableClass, key, key2)) == 0) {
                            n = tieBreakOrder(key, key2);
                        }
                        if ((balanceInsertion = ((n <= 0) ? balanceInsertion.left : balanceInsertion.right)) == null) {
                            break;
                        }
                        continue;
                    }
                    left.parent = balanceInsertion;
                    if (n <= 0) {
                        balanceInsertion.left = left;
                    }
                    else {
                        balanceInsertion.right = left;
                    }
                    balanceInsertion = balanceInsertion(balanceInsertion, left);
                }
            }
            this.root = balanceInsertion;
            assert !(!checkInvariants(this.root));
            return;
        }
        
        private final void lockRoot() {
            if (!ConcurrentHashMap.TreeBin.U.compareAndSwapInt(this, ConcurrentHashMap.TreeBin.LOCKSTATE, 0, 1)) {
                this.contendedLock();
            }
        }
        
        private final void unlockRoot() {
            this.lockState = 0;
        }
        
        private final void contendedLock() {
            int n = 0;
            while (true) {
                int lockState;
                if (((lockState = this.lockState) & 0xFFFFFFFD) == 0x0) {
                    if (ConcurrentHashMap.TreeBin.U.compareAndSwapInt(this, ConcurrentHashMap.TreeBin.LOCKSTATE, lockState, 1)) {
                        break;
                    }
                    continue;
                }
                else if ((lockState & 0x2) == 0x0) {
                    if (!ConcurrentHashMap.TreeBin.U.compareAndSwapInt(this, ConcurrentHashMap.TreeBin.LOCKSTATE, lockState, lockState | 0x2)) {
                        continue;
                    }
                    n = 1;
                    this.waiter = Thread.currentThread();
                }
                else {
                    if (n == 0) {
                        continue;
                    }
                    LockSupport.park(this);
                }
            }
            if (n != 0) {
                this.waiter = null;
            }
        }
        
        @Override
        final ConcurrentHashMap.Node<K, V> find(int n, Object o) {
            if (o != null) {
                ConcurrentHashMap.TreeNode<Object, Object> treeNode = this.first;
                while (treeNode != null) {
                    int lockState;
                    if (((lockState = this.lockState) & 0x3) != 0x0) {
                        Object key;
                        if (treeNode.hash == n && ((key = treeNode.key) == o || (key != null && o.equals(key)))) {
                            return treeNode;
                        }
                        treeNode = treeNode.next;
                    }
                    else {
                        if (ConcurrentHashMap.TreeBin.U.compareAndSwapInt(this, ConcurrentHashMap.TreeBin.LOCKSTATE, lockState, lockState + 4)) {
                            ConcurrentHashMap.TreeNode<Object, Object> treeNode2;
                            try {
                                ConcurrentHashMap.TreeNode<Object, Object> root;
                                treeNode2 = (((root = this.root) == null) ? null : root.findTreeNode(n, o, null));
                                Thread waiter;
                                if (ConcurrentHashMap.TreeBin.U.getAndAddInt(this, ConcurrentHashMap.TreeBin.LOCKSTATE, -4) == 6 && (waiter = this.waiter) != null) {
                                    LockSupport.unpark(waiter);
                                }
                            }
                            finally {
                                Thread waiter2;
                                if (ConcurrentHashMap.TreeBin.U.getAndAddInt(this, ConcurrentHashMap.TreeBin.LOCKSTATE, -4) == 6 && (waiter2 = this.waiter) != null) {
                                    LockSupport.unpark(waiter2);
                                }
                            }
                            return treeNode2;
                        }
                        continue;
                    }
                }
            }
            return null;
        }
        
        final ConcurrentHashMap.TreeNode<K, V> putTreeVal(int n, K k, V v) {
            Class<Object> comparableClass = null;
            int n2 = 0;
            ConcurrentHashMap.TreeNode<Object, Object> root = this.root;
            Label_0345: {
                while (root != null) {
                    int hash;
                    int n3;
                    if ((hash = root.hash) > n) {
                        n3 = -1;
                    }
                    else if (hash < n) {
                        n3 = 1;
                    }
                    else {
                        Object key;
                        if ((key = root.key) == k || (key != null && k.equals(key))) {
                            return root;
                        }
                        if ((comparableClass == null && (comparableClass = ConcurrentHashMap.comparableClassFor(k)) == null) || (n3 = ConcurrentHashMap.compareComparables(comparableClass, k, key)) == 0) {
                            if (n2 == 0) {
                                n2 = 1;
                                ConcurrentHashMap.TreeNode<Object, Object> left;
                                ConcurrentHashMap.TreeNode<Object, Object> treeNode;
                                ConcurrentHashMap.TreeNode<Object, Object> right;
                                if (((left = root.left) != null && (treeNode = left.findTreeNode(n, k, comparableClass)) != null) || ((right = root.right) != null && (treeNode = right.findTreeNode(n, k, comparableClass)) != null)) {
                                    return treeNode;
                                }
                            }
                            n3 = tieBreakOrder(k, key);
                        }
                    }
                    if ((root = ((n3 <= 0) ? root.left : root.right)) == null) {
                        ConcurrentHashMap.TreeNode<Object, Object> first = this.first;
                        Object left2 = this.first = new ConcurrentHashMap.TreeNode(n, k, v, first, root);
                        if (first != null) {
                            first.prev = left2;
                        }
                        if (n3 <= 0) {
                            root.left = left2;
                        }
                        else {
                            root.right = left2;
                        }
                        if (!root.red) {
                            left2.red = true;
                        }
                        else {
                            this.lockRoot();
                            try {
                                this.root = balanceInsertion(this.root, left2);
                            }
                            finally {
                                this.unlockRoot();
                            }
                        }
                        assert !(!checkInvariants(this.root));
                        return null;
                    }
                    else {
                        continue;
                    }
                }
                ConcurrentHashMap.TreeNode treeNode2 = new ConcurrentHashMap.TreeNode(n, k, v);
                this.root = treeNode2;
                this.first = treeNode2;
                continue;
            }
        }
        
        final boolean removeTreeNode(ConcurrentHashMap.TreeNode<K, V> treeNode) {
            ConcurrentHashMap.TreeNode<Object, Object> treeNode2 = (ConcurrentHashMap.TreeNode)treeNode.next;
            ConcurrentHashMap.TreeNode<Object, Object> prev = treeNode.prev;
            if (prev == null) {
                this.first = treeNode2;
            }
            else {
                prev.next = treeNode2;
            }
            if (treeNode2 != null) {
                treeNode2.prev = prev;
            }
            if (this.first == null) {
                this.root = null;
                return true;
            }
            ConcurrentHashMap.TreeNode<K, V> root;
            ConcurrentHashMap.TreeNode<Object, Object> left;
            if ((root = this.root) == null || root.right == null || (left = root.left) == null || left.left == null) {
                return true;
            }
            this.lockRoot();
            try {
                ConcurrentHashMap.TreeNode<Object, Object> left2 = treeNode.left;
                ConcurrentHashMap.TreeNode<Object, Object> right = treeNode.right;
                ConcurrentHashMap.TreeNode<K, V> right2;
                if (left2 != null && right != null) {
                    ConcurrentHashMap.TreeNode<Object, Object> left3;
                    for (; (left3 = right.left) != null; right = left3) {}
                    int red = right.red;
                    right.red = treeNode.red;
                    treeNode.red = red;
                    right2 = right.right;
                    ConcurrentHashMap.TreeNode<Object, Object> parent = treeNode.parent;
                    if (right == right) {
                        treeNode.parent = right;
                        right.right = treeNode;
                    }
                    else {
                        ConcurrentHashMap.TreeNode<Object, Object> parent2 = right.parent;
                        if ((treeNode.parent = parent2) != null) {
                            if (right == parent2.left) {
                                parent2.left = treeNode;
                            }
                            else {
                                parent2.right = treeNode;
                            }
                        }
                        if ((right.right = right) != null) {
                            right.parent = right;
                        }
                    }
                    treeNode.left = null;
                    if ((treeNode.right = right2) != null) {
                        right2.parent = treeNode;
                    }
                    if ((right.left = left2) != null) {
                        left2.parent = right;
                    }
                    if ((right.parent = parent) == null) {
                        root = right;
                    }
                    else if (treeNode == parent.left) {
                        parent.left = right;
                    }
                    else {
                        parent.right = right;
                    }
                    if (right2 != null) {
                    }
                    else {
                        right2 = treeNode;
                    }
                }
                else if (left2 != null) {
                    right2 = left2;
                }
                else if (right != null) {
                    right2 = right;
                }
                else {
                    right2 = treeNode;
                }
                if (right2 != treeNode) {
                    ConcurrentHashMap.TreeNode<K, V> parent3 = treeNode.parent;
                    right2.parent = parent3;
                    ConcurrentHashMap.TreeNode<K, V> treeNode3 = parent3;
                    if (treeNode3 == null) {
                        root = right2;
                    }
                    else if (treeNode == treeNode3.left) {
                        treeNode3.left = right2;
                    }
                    else {
                        treeNode3.right = right2;
                    }
                    ConcurrentHashMap.TreeNode<K, V> left4 = null;
                    treeNode.parent = left4;
                    treeNode.right = left4;
                    treeNode.left = left4;
                }
                this.root = (treeNode.red ? root : balanceDeletion(root, right2));
                ConcurrentHashMap.TreeNode<Object, Object> parent4;
                if (treeNode == right2 && (parent4 = treeNode.parent) != null) {
                    if (treeNode == parent4.left) {
                        parent4.left = null;
                    }
                    else if (treeNode == parent4.right) {
                        parent4.right = null;
                    }
                    treeNode.parent = null;
                }
            }
            finally {
                this.unlockRoot();
            }
            assert !(!checkInvariants(this.root));
            return false;
        }
        
        static <K, V> ConcurrentHashMap.TreeNode<K, V> rotateLeft(ConcurrentHashMap.TreeNode<K, V> treeNode, ConcurrentHashMap.TreeNode<K, V> treeNode2) {
            ConcurrentHashMap.TreeNode<Object, Object> right;
            if (treeNode2 != null && (right = treeNode2.right) != null) {
                ConcurrentHashMap.TreeNode<K, V> left = right.left;
                treeNode2.right = left;
                ConcurrentHashMap.TreeNode<K, V> treeNode3;
                if ((treeNode3 = left) != null) {
                    treeNode3.parent = treeNode2;
                }
                ConcurrentHashMap.TreeNode<K, V> parent = treeNode2.parent;
                right.parent = parent;
                ConcurrentHashMap.TreeNode<K, V> treeNode4;
                if ((treeNode4 = parent) == null) {
                    right.red = false;
                }
                else if (treeNode4.left == treeNode2) {
                    treeNode4.left = right;
                }
                else {
                    treeNode4.right = right;
                }
                right.left = treeNode2;
                treeNode2.parent = right;
                return right;
            }
            else {
                return right;
            }
        }
        
        static <K, V> ConcurrentHashMap.TreeNode<K, V> rotateRight(ConcurrentHashMap.TreeNode<K, V> treeNode, ConcurrentHashMap.TreeNode<K, V> treeNode2) {
            ConcurrentHashMap.TreeNode<Object, Object> left;
            if (treeNode2 != null && (left = treeNode2.left) != null) {
                ConcurrentHashMap.TreeNode<K, V> right = left.right;
                treeNode2.left = right;
                ConcurrentHashMap.TreeNode<K, V> treeNode3;
                if ((treeNode3 = right) != null) {
                    treeNode3.parent = treeNode2;
                }
                ConcurrentHashMap.TreeNode<K, V> parent = treeNode2.parent;
                left.parent = parent;
                ConcurrentHashMap.TreeNode<K, V> treeNode4;
                if ((treeNode4 = parent) == null) {
                    left.red = false;
                }
                else if (treeNode4.right == treeNode2) {
                    treeNode4.right = left;
                }
                else {
                    treeNode4.left = left;
                }
                left.right = treeNode2;
                treeNode2.parent = left;
                return left;
            }
            else {
                return left;
            }
        }
        
        static <K, V> ConcurrentHashMap.TreeNode<K, V> balanceInsertion(ConcurrentHashMap.TreeNode<K, V> treeNode, ConcurrentHashMap.TreeNode<K, V> treeNode2) {
            ConcurrentHashMap.TreeNode<Object, Object> treeNode3;
            treeNode3.red = true;
            while ((treeNode3 = treeNode3.parent) != null) {
                ConcurrentHashMap.TreeNode<K, V> parent;
                if (!treeNode3.red || (parent = treeNode3.parent) == null) {
                    return treeNode;
                }
                ConcurrentHashMap.TreeNode<Object, Object> left;
                if (treeNode3 == (left = parent.left)) {
                    ConcurrentHashMap.TreeNode<Object, Object> right;
                    if ((right = parent.right) != null && right.red) {
                        right.red = false;
                        treeNode3.red = false;
                        parent.red = true;
                        treeNode2 = parent;
                    }
                    else {
                        if (treeNode3 == treeNode3.right) {
                            treeNode = rotateLeft(treeNode, treeNode3);
                            parent = (((treeNode3 = treeNode3.parent) == null) ? null : treeNode3.parent);
                        }
                        if (treeNode3 == null) {
                            continue;
                        }
                        treeNode3.red = false;
                        if (parent == null) {
                            continue;
                        }
                        parent.red = true;
                        treeNode = rotateRight(treeNode, parent);
                    }
                }
                else if (left != null && left.red) {
                    left.red = false;
                    treeNode3.red = false;
                    parent.red = true;
                    treeNode2 = parent;
                }
                else {
                    if (treeNode3 == treeNode3.left) {
                        treeNode = rotateRight(treeNode, treeNode3);
                        parent = (((treeNode3 = treeNode3.parent) == null) ? null : treeNode3.parent);
                    }
                    if (treeNode3 == null) {
                        continue;
                    }
                    treeNode3.red = false;
                    if (parent == null) {
                        continue;
                    }
                    parent.red = true;
                    treeNode = rotateLeft(treeNode, parent);
                }
            }
            treeNode3.red = false;
            return treeNode3;
        }
        
        static <K, V> ConcurrentHashMap.TreeNode<K, V> balanceDeletion(ConcurrentHashMap.TreeNode<K, V> node, ConcurrentHashMap.TreeNode<K, V> treeNode) {
            while (treeNode != null && treeNode != node) {
                ConcurrentHashMap.TreeNode<K, V> treeNode2;
                if ((treeNode2 = treeNode.parent) == null) {
                    treeNode.red = false;
                    return treeNode;
                }
                if (treeNode.red) {
                    treeNode.red = false;
                    return node;
                }
                ConcurrentHashMap.TreeNode<K, V> left;
                if ((left = treeNode2.left) == treeNode) {
                    ConcurrentHashMap.TreeNode<K, V> right2;
                    if ((right2 = treeNode2.right) != null && right2.red) {
                        right2.red = false;
                        treeNode2.red = true;
                        node = rotateLeft(node, treeNode2);
                        right2 = (((treeNode2 = treeNode.parent) == null) ? null : treeNode2.right);
                    }
                    if (right2 == null) {
                        treeNode = treeNode2;
                    }
                    else {
                        ConcurrentHashMap.TreeNode<Object, Object> left4 = right2.left;
                        ConcurrentHashMap.TreeNode<Object, Object> right3 = right2.right;
                        if ((right3 == null || !right3.red) && (left4 == null || !left4.red)) {
                            right2.red = true;
                            treeNode = treeNode2;
                        }
                        else {
                            if (right3 == null || !right3.red) {
                                if (left4 != null) {
                                    left4.red = false;
                                }
                                right2.red = true;
                                node = rotateRight(node, right2);
                                right2 = (((treeNode2 = treeNode.parent) == null) ? null : treeNode2.right);
                            }
                            if (right2 != null) {
                                right2.red = (treeNode2 != null && treeNode2.red);
                                ConcurrentHashMap.TreeNode<Object, Object> right4;
                                if ((right4 = right2.right) != null) {
                                    right4.red = false;
                                }
                            }
                            if (treeNode2 != null) {
                                treeNode2.red = false;
                                node = rotateLeft(node, treeNode2);
                            }
                            treeNode = node;
                        }
                    }
                }
                else {
                    if (left != null && left.red) {
                        left.red = false;
                        treeNode2.red = true;
                        node = rotateRight(node, treeNode2);
                        left = (((treeNode2 = treeNode.parent) == null) ? null : treeNode2.left);
                    }
                    if (left == null) {
                        treeNode = treeNode2;
                    }
                    else {
                        ConcurrentHashMap.TreeNode<Object, Object> left2 = left.left;
                        ConcurrentHashMap.TreeNode<Object, Object> right = left.right;
                        if ((left2 == null || !left2.red) && (right == null || !right.red)) {
                            left.red = true;
                            treeNode = treeNode2;
                        }
                        else {
                            if (left2 == null || !left2.red) {
                                if (right != null) {
                                    right.red = false;
                                }
                                left.red = true;
                                node = rotateLeft(node, left);
                                left = (((treeNode2 = treeNode.parent) == null) ? null : treeNode2.left);
                            }
                            if (left != null) {
                                left.red = (treeNode2 != null && treeNode2.red);
                                ConcurrentHashMap.TreeNode<Object, Object> left3;
                                if ((left3 = left.left) != null) {
                                    left3.red = false;
                                }
                            }
                            if (treeNode2 != null) {
                                treeNode2.red = false;
                                node = rotateRight(node, treeNode2);
                            }
                            treeNode = node;
                        }
                    }
                }
            }
            return node;
        }
        
        static <K, V> boolean checkInvariants(ConcurrentHashMap.TreeNode<K, V> treeNode) {
            ConcurrentHashMap.TreeNode<Object, Object> parent = treeNode.parent;
            ConcurrentHashMap.TreeNode<Object, Object> left = treeNode.left;
            ConcurrentHashMap.TreeNode<Object, Object> right = treeNode.right;
            ConcurrentHashMap.TreeNode<Object, Object> prev = treeNode.prev;
            ConcurrentHashMap.TreeNode<Object, Object> treeNode2 = (ConcurrentHashMap.TreeNode)treeNode.next;
            return (prev == null || prev.next == treeNode) && (treeNode2 == null || treeNode2.prev == treeNode) && (parent == null || treeNode == parent.left || treeNode == parent.right) && (left == null || (left.parent == treeNode && left.hash <= treeNode.hash)) && (right == null || (right.parent == treeNode && right.hash >= treeNode.hash)) && (!treeNode.red || left == null || !left.red || right == null || !right.red) && (left == null || checkInvariants(left)) && (right == null || checkInvariants(right));
        }
        
        static {
            try {
                U = Unsafe.getUnsafe();
                LOCKSTATE = ConcurrentHashMap.TreeBin.U.objectFieldOffset(ConcurrentHashMap.TreeBin.class.getDeclaredField("lockState"));
            }
            catch (Exception cause) {
                throw new Error(cause);
            }
        }
    }
    
    static final class TableStack<K, V>
    {
        int length;
        int index;
        ConcurrentHashMap.Node<K, V>[] tab;
        ConcurrentHashMap.TableStack<K, V> next;
    }
    
    static class Traverser<K, V>
    {
        ConcurrentHashMap.Node<K, V>[] tab;
        ConcurrentHashMap.Node<K, V> next;
        ConcurrentHashMap.TableStack<K, V> stack;
        ConcurrentHashMap.TableStack<K, V> spare;
        int index;
        int baseIndex;
        int baseLimit;
        final int baseSize;
        
        Traverser(ConcurrentHashMap.Node<K, V>[] tab, int baseSize, int n, int baseLimit) {
            this.tab = tab;
            this.baseSize = baseSize;
            this.index = n;
            this.baseIndex = n;
            this.baseLimit = baseLimit;
            this.next = null;
        }
        
        final ConcurrentHashMap.Node<K, V> advance() {
            ConcurrentHashMap.Node<K, V> next;
            if ((next = this.next) != null) {
                next = next.next;
            }
            while (next == null) {
                ConcurrentHashMap.Node[] tab;
                int length;
                int index;
                if (this.baseIndex >= this.baseLimit || (tab = this.tab) == null || (length = tab.length) <= (index = this.index) || index < 0) {
                    return this.next = null;
                }
                if ((next = ConcurrentHashMap.tabAt(tab, index)) != null && next.hash < 0) {
                    if ((next instanceof ConcurrentHashMap.ForwardingNode)) {
                        this.tab = ((ConcurrentHashMap.ForwardingNode)next).nextTable;
                        next = null;
                        this.pushState(tab, index, length);
                        continue;
                    }
                    if ((next instanceof ConcurrentHashMap.TreeBin)) {
                        next = ((ConcurrentHashMap.TreeBin)next).first;
                    }
                    else {
                        next = null;
                    }
                }
                if (this.stack != null) {
                    this.recoverState(length);
                }
                else if ((this.index = index + this.baseSize) >= length) {
                    this.index = ++this.baseIndex;
                }
                else {
                    continue;
                }
            }
            return this.next = next;
        }
        
        private void pushState(ConcurrentHashMap.Node<K, V>[] tab, int index, int length) {
            ConcurrentHashMap.TableStack<Object, Object> spare = this.spare;
            if (spare != null) {
                this.spare = spare.next;
            }
            else {
                spare = new ConcurrentHashMap.TableStack();
            }
            spare.tab = tab;
            spare.length = length;
            spare.index = index;
            spare.next = this.stack;
            this.stack = spare;
        }
        
        private void recoverState(int n) {
            ConcurrentHashMap.TableStack<Object, Object> stack;
            int length;
            while ((stack = this.stack) != null && (this.index += (length = stack.length)) >= n) {
                n = length;
                this.index = stack.index;
                this.tab = stack.tab;
                stack.tab = null;
                ConcurrentHashMap.TableStack<Object, Object> next = stack.next;
                stack.next = this.spare;
                this.stack = next;
                this.spare = stack;
            }
            if (stack == null && (this.index += this.baseSize) >= n) {
                this.index = ++this.baseIndex;
            }
        }
    }
    
    static class BaseIterator<K, V> extends ConcurrentHashMap.Traverser<K, V>
    {
        final ConcurrentHashMap<K, V> map;
        ConcurrentHashMap.Node<K, V> lastReturned;
        
        BaseIterator(ConcurrentHashMap.Node<K, V>[] array, int n, int n2, int n3, ConcurrentHashMap<K, V> map) {
            super(array, n, n2, n3);
            this.map = map;
            this.advance();
        }
        
        public final boolean hasNext() {
            return this.next != null;
        }
        
        public final boolean hasMoreElements() {
            return this.next != null;
        }
        
        public final void remove() {
            ConcurrentHashMap.Node<Object, Object> lastReturned;
            if ((lastReturned = this.lastReturned) == null) {
                throw new IllegalStateException();
            }
            this.lastReturned = null;
            this.map.replaceNode(lastReturned.key, null, null);
            return;
        }
    }
    
    static final class KeyIterator<K, V> extends ConcurrentHashMap.BaseIterator<K, V> implements Iterator<K>, Enumeration<K>
    {
        KeyIterator(ConcurrentHashMap.Node<K, V>[] array, int n, int n2, int n3, ConcurrentHashMap<K, V> concurrentHashMap) {
            super(array, n, n2, n3, concurrentHashMap);
        }
        
        @Override
        public final K next() {
            ConcurrentHashMap.Node<Object, Object> next;
            if ((next = this.next) == null) {
                throw new NoSuchElementException();
            }
            Object key = next.key;
            this.lastReturned = next;
            this.advance();
            return key;
        }
        
        @Override
        public final K nextElement() {
            return this.next();
        }
    }
    
    static final class ValueIterator<K, V> extends ConcurrentHashMap.BaseIterator<K, V> implements Iterator<V>, Enumeration<V>
    {
        ValueIterator(ConcurrentHashMap.Node<K, V>[] array, int n, int n2, int n3, ConcurrentHashMap<K, V> concurrentHashMap) {
            super(array, n, n2, n3, concurrentHashMap);
        }
        
        @Override
        public final V next() {
            ConcurrentHashMap.Node<Object, Object> next;
            if ((next = this.next) == null) {
                throw new NoSuchElementException();
            }
            Object val = next.val;
            this.lastReturned = next;
            this.advance();
            return val;
        }
        
        @Override
        public final V nextElement() {
            return this.next();
        }
    }
    
    static final class EntryIterator<K, V> extends ConcurrentHashMap.BaseIterator<K, V> implements Iterator<Map.Entry<K, V>>
    {
        EntryIterator(ConcurrentHashMap.Node<K, V>[] array, int n, int n2, int n3, ConcurrentHashMap<K, V> concurrentHashMap) {
            super(array, n, n2, n3, concurrentHashMap);
        }
        
        @Override
        public final Map.Entry<K, V> next() {
            ConcurrentHashMap.Node<Object, Object> next;
            if ((next = this.next) == null) {
                throw new NoSuchElementException();
            }
            Object key = next.key;
            Object val = next.val;
            this.lastReturned = next;
            this.advance();
            return new ConcurrentHashMap.MapEntry(key, val, this.map);
        }
    }
    
    static final class MapEntry<K, V> implements Map.Entry<K, V>
    {
        final K key;
        V val;
        final ConcurrentHashMap<K, V> map;
        
        MapEntry(K key, V val, ConcurrentHashMap<K, V> map) {
            this.key = key;
            this.val = val;
            this.map = map;
        }
        
        @Override
        public K getKey() {
            return this.key;
        }
        
        @Override
        public V getValue() {
            return this.val;
        }
        
        @Override
        public int hashCode() {
            return this.key.hashCode() ^ this.val.hashCode();
        }
        
        @Override
        public String toString() {
            return this.key + "=" + this.val;
        }
        
        @Override
        public boolean equals(Object o) {
            Map.Entry<Object, Object> entry;
            Object key;
            Object value;
            return o instanceof Map.Entry && (key = (entry = (Map.Entry)o).getKey()) != null && (value = entry.getValue()) != null && (key == this.key || key.equals(this.key)) && (value == this.val || value.equals(this.val));
        }
        
        @Override
        public V setValue(V val) {
            if (val == null) {
                throw new NullPointerException();
            }
            Object val2 = this.val;
            this.val = val;
            this.map.put(this.key, val);
            return val2;
        }
    }
    
    static final class KeySpliterator<K, V> extends ConcurrentHashMap.Traverser<K, V> implements Spliterator<K>
    {
        long est;
        
        KeySpliterator(ConcurrentHashMap.Node<K, V>[] array, int n, int n2, int n3, long est) {
            super(array, n, n2, n3);
            this.est = est;
        }
        
        @Override
        public Spliterator<K> trySplit() {
            int baseIndex = this.baseIndex;
            int baseLimit;
            int n;
            if ((n = baseIndex + (baseLimit = this.baseLimit) >>> 1) <= baseIndex) {
                return null;
            }
            ConcurrentHashMap.Node<K, V>[] tab;
            int baseSize;
            int baseLimit2;
            ConcurrentHashMap.KeySpliterator keySpliterator = new ConcurrentHashMap.KeySpliterator(tab, baseSize, baseLimit2, baseLimit, this.est >>>= 1);
            tab = this.tab;
            baseSize = this.baseSize;
            baseLimit2 = n;
            this.baseLimit = baseLimit2;
            return keySpliterator;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            ConcurrentHashMap.Node<K, V> advance;
            while ((advance = this.advance()) != null) {
                consumer.accept(advance.key);
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            ConcurrentHashMap.Node<K, V> advance;
            if ((advance = this.advance()) == null) {
                return false;
            }
            consumer.accept(advance.key);
            return true;
        }
        
        @Override
        public long estimateSize() {
            return this.est;
        }
        
        @Override
        public int characteristics() {
            return 4353;
        }
    }
    
    static final class ValueSpliterator<K, V> extends ConcurrentHashMap.Traverser<K, V> implements Spliterator<V>
    {
        long est;
        
        ValueSpliterator(ConcurrentHashMap.Node<K, V>[] array, int n, int n2, int n3, long est) {
            super(array, n, n2, n3);
            this.est = est;
        }
        
        @Override
        public Spliterator<V> trySplit() {
            int baseIndex = this.baseIndex;
            int baseLimit;
            int n;
            if ((n = baseIndex + (baseLimit = this.baseLimit) >>> 1) <= baseIndex) {
                return null;
            }
            ConcurrentHashMap.Node<K, V>[] tab;
            int baseSize;
            int baseLimit2;
            ConcurrentHashMap.ValueSpliterator valueSpliterator = new ConcurrentHashMap.ValueSpliterator(tab, baseSize, baseLimit2, baseLimit, this.est >>>= 1);
            tab = this.tab;
            baseSize = this.baseSize;
            baseLimit2 = n;
            this.baseLimit = baseLimit2;
            return valueSpliterator;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super V> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            ConcurrentHashMap.Node<K, V> advance;
            while ((advance = this.advance()) != null) {
                consumer.accept(advance.val);
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super V> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            ConcurrentHashMap.Node<K, V> advance;
            if ((advance = this.advance()) == null) {
                return false;
            }
            consumer.accept(advance.val);
            return true;
        }
        
        @Override
        public long estimateSize() {
            return this.est;
        }
        
        @Override
        public int characteristics() {
            return 4352;
        }
    }
    
    static final class EntrySpliterator<K, V> extends ConcurrentHashMap.Traverser<K, V> implements Spliterator<Map.Entry<K, V>>
    {
        final ConcurrentHashMap<K, V> map;
        long est;
        
        EntrySpliterator(ConcurrentHashMap.Node<K, V>[] array, int n, int n2, int n3, long est, ConcurrentHashMap<K, V> map) {
            super(array, n, n2, n3);
            this.map = map;
            this.est = est;
        }
        
        @Override
        public Spliterator<Map.Entry<K, V>> trySplit() {
            int baseIndex = this.baseIndex;
            int baseLimit;
            int n;
            if ((n = baseIndex + (baseLimit = this.baseLimit) >>> 1) <= baseIndex) {
                return null;
            }
            ConcurrentHashMap.Node<K, V>[] tab;
            int baseSize;
            int baseLimit2;
            ConcurrentHashMap.EntrySpliterator entrySpliterator = new ConcurrentHashMap.EntrySpliterator(tab, baseSize, baseLimit2, baseLimit, this.est >>>= 1, this.map);
            tab = this.tab;
            baseSize = this.baseSize;
            baseLimit2 = n;
            this.baseLimit = baseLimit2;
            return entrySpliterator;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super Map.Entry<K, V>> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            ConcurrentHashMap.Node<K, V> advance;
            while ((advance = this.advance()) != null) {
                consumer.accept(new ConcurrentHashMap.MapEntry(advance.key, advance.val, this.map));
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super Map.Entry<K, V>> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            ConcurrentHashMap.Node<K, V> advance;
            if ((advance = this.advance()) == null) {
                return false;
            }
            consumer.accept(new ConcurrentHashMap.MapEntry(advance.key, advance.val, this.map));
            return true;
        }
        
        @Override
        public long estimateSize() {
            return this.est;
        }
        
        @Override
        public int characteristics() {
            return 4353;
        }
    }
    
    abstract static class CollectionView<K, V, E> implements Collection<E>, Serializable
    {
        private static final long serialVersionUID = 7249069246763182397L;
        final ConcurrentHashMap<K, V> map;
        private static final String oomeMsg = "Required array size too large";
        
        CollectionView(ConcurrentHashMap<K, V> map) {
            this.map = map;
        }
        
        public ConcurrentHashMap<K, V> getMap() {
            return this.map;
        }
        
        @Override
        public final void clear() {
            this.map.clear();
        }
        
        @Override
        public final int size() {
            return this.map.size();
        }
        
        @Override
        public final boolean isEmpty() {
            return this.map.isEmpty();
        }
        
        @Override
        public abstract Iterator<E> iterator();
        
        @Override
        public abstract boolean contains(Object p0);
        
        @Override
        public abstract boolean remove(Object p0);
        
        @Override
        public final Object[] toArray() {
            long mappingCount = this.map.mappingCount();
            if (mappingCount > 2147483639L) {
                throw new OutOfMemoryError("Required array size too large");
            }
            int newLength = (int)mappingCount;
            Object[] copy = new Object[newLength];
            int newLength2 = 0;
            for (Object next : this) {
                if (newLength2 == newLength) {
                    if (newLength >= 2147483639) {
                        throw new OutOfMemoryError("Required array size too large");
                    }
                    if (newLength >= 1073741819) {
                        newLength = 2147483639;
                    }
                    else {
                        newLength += (newLength >>> 1) + 1;
                    }
                    copy = Arrays.copyOf(copy, newLength);
                }
                copy[newLength2++] = next;
            }
            return (newLength2 == newLength) ? copy : Arrays.copyOf(copy, newLength2);
        }
        
        @Override
        public final <T> T[] toArray(T[] array) {
            long mappingCount = this.map.mappingCount();
            if (mappingCount > 2147483639L) {
                throw new OutOfMemoryError("Required array size too large");
            }
            int length = (int)mappingCount;
            Object[] copy = (array.length >= length) ? array : ((Object[])(Object[])Array.newInstance(array.getClass().getComponentType(), length));
            int length2 = copy.length;
            int newLength = 0;
            for (Object next : this) {
                if (newLength == length2) {
                    if (length2 >= 2147483639) {
                        throw new OutOfMemoryError("Required array size too large");
                    }
                    if (length2 >= 1073741819) {
                        length2 = 2147483639;
                    }
                    else {
                        length2 += (length2 >>> 1) + 1;
                    }
                    copy = Arrays.copyOf(copy, length2);
                }
                copy[newLength++] = next;
            }
            if (array == copy && newLength < length2) {
                copy[newLength] = null;
                return copy;
            }
            return (newLength == length2) ? copy : Arrays.copyOf(copy, newLength);
        }
        
        @Override
        public final String toString() {
            StringBuilder sb = new StringBuilder();
            sb.append('[');
            Iterator<E> iterator = this.iterator();
            if (iterator.hasNext()) {
                while (true) {
                    Object next = iterator.next();
                    sb.append((next == this) ? "(this Collection)" : next);
                    if (!iterator.hasNext()) {
                        break;
                    }
                    sb.append(',').append(' ');
                }
            }
            return sb.append(']').toString();
        }
        
        @Override
        public final boolean containsAll(Collection<?> collection) {
            if (collection != this) {
                for (Object next : collection) {
                    if (next == null || !this.contains(next)) {
                        return false;
                    }
                    continue;
                }
            }
            return true;
        }
        
        @Override
        public final boolean removeAll(Collection<?> collection) {
            if (collection == null) {
                throw new NullPointerException();
            }
            boolean n = false;
            Iterator<E> iterator = this.iterator();
            while (iterator.hasNext()) {
                if (!collection.contains(iterator.next())) {
                    continue;
                }
                iterator.remove();
                n = true;
            }
            return n;
        }
        
        @Override
        public final boolean retainAll(Collection<?> collection) {
            if (collection == null) {
                throw new NullPointerException();
            }
            boolean n = false;
            Iterator<E> iterator = this.iterator();
            while (iterator.hasNext()) {
                if (collection.contains(iterator.next())) {
                    continue;
                }
                iterator.remove();
                n = true;
            }
            return n;
        }
    }
    
    public static class KeySetView<K, V> extends ConcurrentHashMap.CollectionView<K, V, K> implements Set<K>, Serializable
    {
        private static final long serialVersionUID = 7249069246763182397L;
        private final V value;
        
        KeySetView(ConcurrentHashMap<K, V> concurrentHashMap, V value) {
            super(concurrentHashMap);
            this.value = value;
        }
        
        public V getMappedValue() {
            return this.value;
        }
        
        @Override
        public boolean contains(Object o) {
            return this.map.containsKey(o);
        }
        
        @Override
        public boolean remove(Object o) {
            return this.map.remove(o) != null;
        }
        
        @Override
        public Iterator<K> iterator() {
            ConcurrentHashMap<Object, Object> map = this.map;
            ConcurrentHashMap.Node[] table;
            int n = ((table = map.table) == null) ? 0 : table.length;
            return new ConcurrentHashMap.KeyIterator(table, n, 0, n, map);
        }
        
        @Override
        public boolean add(K k) {
            Object value;
            if ((value = this.value) == null) {
                throw new UnsupportedOperationException();
            }
            return this.map.putVal(k, value, true) == null;
        }
        
        @Override
        public boolean addAll(Collection<? extends K> collection) {
            boolean n = false;
            Object value;
            if ((value = this.value) == null) {
                throw new UnsupportedOperationException();
            }
            Iterator<Object> iterator = collection.iterator();
            while (iterator.hasNext()) {
                if (this.map.putVal(iterator.next(), value, true) == null) {
                    n = true;
                }
                else {
                    continue;
                }
            }
            return n;
        }
        
        @Override
        public int hashCode() {
            int n = 0;
            Iterator<K> iterator = this.iterator();
            while (iterator.hasNext()) {
                n += iterator.next().hashCode();
            }
            return n;
        }
        
        @Override
        public boolean equals(Object o) {
            Set<Object> set;
            return o instanceof Set && ((set = (Set)o) == this || (this.containsAll(set) && set.containsAll(this)));
        }
        
        @Override
        public Spliterator<K> spliterator() {
            ConcurrentHashMap<Object, Object> map = this.map;
            long sumCount = map.sumCount();
            ConcurrentHashMap.Node[] table;
            int n = ((table = map.table) == null) ? 0 : table.length;
            return new ConcurrentHashMap.KeySpliterator(table, n, 0, n, (sumCount < 0L) ? 0L : sumCount);
        }
        
        @Override
        public void forEach(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            ConcurrentHashMap.Node[] table;
            if ((table = this.map.table) != null) {
                ConcurrentHashMap.Node<Object, Object> advance;
                while ((advance = new ConcurrentHashMap.Traverser(table, table.length, 0, table.length).advance()) != null) {
                    consumer.accept(advance.key);
                }
            }
            return;
        }
    }
    
    static final class ValuesView<K, V> extends ConcurrentHashMap.CollectionView<K, V, V> implements Collection<V>, Serializable
    {
        private static final long serialVersionUID = 2249069246763182397L;
        
        ValuesView(ConcurrentHashMap<K, V> concurrentHashMap) {
            super(concurrentHashMap);
        }
        
        @Override
        public final boolean contains(Object o) {
            return this.map.containsValue(o);
        }
        
        @Override
        public final boolean remove(Object o) {
            if (o != null) {
                Iterator<V> iterator = this.iterator();
                while (iterator.hasNext()) {
                    if (!o.equals(iterator.next())) {
                        continue;
                    }
                    iterator.remove();
                    return true;
                }
            }
            return false;
        }
        
        @Override
        public final Iterator<V> iterator() {
            ConcurrentHashMap<Object, Object> map = this.map;
            ConcurrentHashMap.Node[] table;
            int n = ((table = map.table) == null) ? 0 : table.length;
            return new ConcurrentHashMap.ValueIterator(table, n, 0, n, map);
        }
        
        @Override
        public final boolean add(V v) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public final boolean addAll(Collection<? extends V> collection) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public Spliterator<V> spliterator() {
            ConcurrentHashMap<Object, Object> map = this.map;
            long sumCount = map.sumCount();
            ConcurrentHashMap.Node[] table;
            int n = ((table = map.table) == null) ? 0 : table.length;
            return new ConcurrentHashMap.ValueSpliterator(table, n, 0, n, (sumCount < 0L) ? 0L : sumCount);
        }
        
        @Override
        public void forEach(Consumer<? super V> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            ConcurrentHashMap.Node[] table;
            if ((table = this.map.table) != null) {
                ConcurrentHashMap.Node<Object, Object> advance;
                while ((advance = new ConcurrentHashMap.Traverser(table, table.length, 0, table.length).advance()) != null) {
                    consumer.accept(advance.val);
                }
            }
            return;
        }
    }
    
    static final class EntrySetView<K, V> extends ConcurrentHashMap.CollectionView<K, V, Map.Entry<K, V>> implements Set<Map.Entry<K, V>>, Serializable
    {
        private static final long serialVersionUID = 2249069246763182397L;
        
        EntrySetView(ConcurrentHashMap<K, V> concurrentHashMap) {
            super(concurrentHashMap);
        }
        
        @Override
        public boolean contains(Object o) {
            Map.Entry<Object, Object> entry;
            Object key;
            Object value;
            Object value2;
            return o instanceof Map.Entry && (key = (entry = (Map.Entry)o).getKey()) != null && (value = this.map.get(key)) != null && (value2 = entry.getValue()) != null && (value2 == value || value2.equals(value));
        }
        
        @Override
        public boolean remove(Object o) {
            Map.Entry<Object, Object> entry;
            Object key;
            Object value;
            return o instanceof Map.Entry && (key = (entry = (Map.Entry)o).getKey()) != null && (value = entry.getValue()) != null && this.map.remove(key, value);
        }
        
        @Override
        public Iterator<Map.Entry<K, V>> iterator() {
            ConcurrentHashMap<Object, Object> map = this.map;
            ConcurrentHashMap.Node[] table;
            int n = ((table = map.table) == null) ? 0 : table.length;
            return new ConcurrentHashMap.EntryIterator(table, n, 0, n, map);
        }
        
        @Override
        public boolean add(Map.Entry<K, V> entry) {
            return this.map.putVal(entry.getKey(), entry.getValue(), false) == null;
        }
        
        @Override
        public boolean addAll(Collection<? extends Map.Entry<K, V>> collection) {
            boolean n = false;
            Iterator<Object> iterator = collection.iterator();
            while (iterator.hasNext()) {
                if (this.add((Map.Entry)iterator.next())) {
                    n = true;
                }
                else {
                    continue;
                }
            }
            return n;
        }
        
        @Override
        public final int hashCode() {
            int n = 0;
            ConcurrentHashMap.Node[] table;
            if ((table = this.map.table) != null) {
                ConcurrentHashMap.Node<Object, Object> advance;
                while ((advance = new ConcurrentHashMap.Traverser(table, table.length, 0, table.length).advance()) != null) {
                    n += advance.hashCode();
                }
            }
            return n;
        }
        
        @Override
        public final boolean equals(Object o) {
            Set<Object> set;
            return o instanceof Set && ((set = (Set)o) == this || (this.containsAll(set) && set.containsAll(this)));
        }
        
        @Override
        public Spliterator<Map.Entry<K, V>> spliterator() {
            ConcurrentHashMap<Object, Object> map = this.map;
            long sumCount = map.sumCount();
            ConcurrentHashMap.Node[] table;
            int n = ((table = map.table) == null) ? 0 : table.length;
            return new ConcurrentHashMap.EntrySpliterator(table, n, 0, n, (sumCount < 0L) ? 0L : sumCount, map);
        }
        
        @Override
        public void forEach(Consumer<? super Map.Entry<K, V>> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            ConcurrentHashMap.Node[] table;
            if ((table = this.map.table) != null) {
                ConcurrentHashMap.Node<Object, Object> advance;
                while ((advance = new ConcurrentHashMap.Traverser(table, table.length, 0, table.length).advance()) != null) {
                    consumer.accept(new ConcurrentHashMap.MapEntry(advance.key, advance.val, this.map));
                }
            }
            return;
        }
    }
    
    abstract static class BulkTask<K, V, R> extends CountedCompleter<R>
    {
        ConcurrentHashMap.Node<K, V>[] tab;
        ConcurrentHashMap.Node<K, V> next;
        ConcurrentHashMap.TableStack<K, V> stack;
        ConcurrentHashMap.TableStack<K, V> spare;
        int index;
        int baseIndex;
        int baseLimit;
        final int baseSize;
        int batch;
        
        BulkTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int batch, int n, int baseLimit, ConcurrentHashMap.Node<K, V>[] tab) {
            super(bulkTask);
            this.batch = batch;
            this.baseIndex = n;
            this.index = n;
            this.tab = tab;
            if (tab == null) {
                int n2 = 0;
                this.baseLimit = n2;
                this.baseSize = n2;
            }
            else if (bulkTask == null) {
                int length = tab.length;
                this.baseLimit = length;
                this.baseSize = length;
            }
            else {
                this.baseLimit = baseLimit;
                this.baseSize = bulkTask.baseSize;
            }
        }
        
        final ConcurrentHashMap.Node<K, V> advance() {
            ConcurrentHashMap.Node<K, V> next;
            if ((next = this.next) != null) {
                next = next.next;
            }
            while (next == null) {
                ConcurrentHashMap.Node[] tab;
                int length;
                int index;
                if (this.baseIndex >= this.baseLimit || (tab = this.tab) == null || (length = tab.length) <= (index = this.index) || index < 0) {
                    return this.next = null;
                }
                if ((next = ConcurrentHashMap.tabAt(tab, index)) != null && next.hash < 0) {
                    if ((next instanceof ConcurrentHashMap.ForwardingNode)) {
                        this.tab = ((ConcurrentHashMap.ForwardingNode)next).nextTable;
                        next = null;
                        this.pushState(tab, index, length);
                        continue;
                    }
                    if ((next instanceof ConcurrentHashMap.TreeBin)) {
                        next = ((ConcurrentHashMap.TreeBin)next).first;
                    }
                    else {
                        next = null;
                    }
                }
                if (this.stack != null) {
                    this.recoverState(length);
                }
                else if ((this.index = index + this.baseSize) >= length) {
                    this.index = ++this.baseIndex;
                }
                else {
                    continue;
                }
            }
            return this.next = next;
        }
        
        private void pushState(ConcurrentHashMap.Node<K, V>[] tab, int index, int length) {
            ConcurrentHashMap.TableStack<Object, Object> spare = this.spare;
            if (spare != null) {
                this.spare = spare.next;
            }
            else {
                spare = new ConcurrentHashMap.TableStack();
            }
            spare.tab = tab;
            spare.length = length;
            spare.index = index;
            spare.next = this.stack;
            this.stack = spare;
        }
        
        private void recoverState(int n) {
            ConcurrentHashMap.TableStack<Object, Object> stack;
            int length;
            while ((stack = this.stack) != null && (this.index += (length = stack.length)) >= n) {
                n = length;
                this.index = stack.index;
                this.tab = stack.tab;
                stack.tab = null;
                ConcurrentHashMap.TableStack<Object, Object> next = stack.next;
                stack.next = this.spare;
                this.stack = next;
                this.spare = stack;
            }
            if (stack == null && (this.index += this.baseSize) >= n) {
                this.index = ++this.baseIndex;
            }
        }
    }
    
    static final class ForEachKeyTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Void>
    {
        final Consumer<? super K> action;
        
        ForEachKeyTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, Consumer<? super K> action) {
            super(bulkTask, n, n2, n3, array);
            this.action = action;
        }
        
        @Override
        public final void compute() {
            Consumer<ConcurrentHashMap.Node> action;
            if ((action = this.action) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    new ConcurrentHashMap.ForEachKeyTask(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, action).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    action.accept(advance.key);
                }
                this.propagateCompletion();
            }
        }
    }
    
    static final class ForEachValueTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Void>
    {
        final Consumer<? super V> action;
        
        ForEachValueTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, Consumer<? super V> action) {
            super(bulkTask, n, n2, n3, array);
            this.action = action;
        }
        
        @Override
        public final void compute() {
            Consumer<ConcurrentHashMap.Node> action;
            if ((action = this.action) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    new ConcurrentHashMap.ForEachValueTask(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, action).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    action.accept(advance.val);
                }
                this.propagateCompletion();
            }
        }
    }
    
    static final class ForEachEntryTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Void>
    {
        final Consumer<? super Map.Entry<K, V>> action;
        
        ForEachEntryTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, Consumer<? super Map.Entry<K, V>> action) {
            super(bulkTask, n, n2, n3, array);
            this.action = action;
        }
        
        @Override
        public final void compute() {
            Consumer<ConcurrentHashMap.Node> action;
            if ((action = this.action) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    new ConcurrentHashMap.ForEachEntryTask(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, action).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    action.accept(advance);
                }
                this.propagateCompletion();
            }
        }
    }
    
    static final class ForEachMappingTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Void>
    {
        final BiConsumer<? super K, ? super V> action;
        
        ForEachMappingTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, BiConsumer<? super K, ? super V> action) {
            super(bulkTask, n, n2, n3, array);
            this.action = action;
        }
        
        @Override
        public final void compute() {
            BiConsumer<ConcurrentHashMap.Node, ConcurrentHashMap.Node> action;
            if ((action = this.action) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    new ConcurrentHashMap.ForEachMappingTask(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, action).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    action.accept(advance.key, advance.val);
                }
                this.propagateCompletion();
            }
        }
    }
    
    static final class ForEachTransformedKeyTask<K, V, U> extends ConcurrentHashMap.BulkTask<K, V, Void>
    {
        final Function<? super K, ? extends U> transformer;
        final Consumer<? super U> action;
        
        ForEachTransformedKeyTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, Function<? super K, ? extends U> transformer, Consumer<? super U> action) {
            super(bulkTask, n, n2, n3, array);
            this.transformer = transformer;
            this.action = action;
        }
        
        @Override
        public final void compute() {
            Function<ConcurrentHashMap.Node, Object> transformer;
            Consumer<Object> action;
            if ((transformer = this.transformer) != null && (action = this.action) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    new ConcurrentHashMap.ForEachTransformedKeyTask(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, transformer, action).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    Object apply;
                    if ((apply = transformer.apply(advance.key)) != null) {
                        action.accept(apply);
                    }
                    else {
                        continue;
                    }
                }
                this.propagateCompletion();
            }
        }
    }
    
    static final class ForEachTransformedValueTask<K, V, U> extends ConcurrentHashMap.BulkTask<K, V, Void>
    {
        final Function<? super V, ? extends U> transformer;
        final Consumer<? super U> action;
        
        ForEachTransformedValueTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, Function<? super V, ? extends U> transformer, Consumer<? super U> action) {
            super(bulkTask, n, n2, n3, array);
            this.transformer = transformer;
            this.action = action;
        }
        
        @Override
        public final void compute() {
            Function<ConcurrentHashMap.Node, Object> transformer;
            Consumer<Object> action;
            if ((transformer = this.transformer) != null && (action = this.action) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    new ConcurrentHashMap.ForEachTransformedValueTask(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, transformer, action).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    Object apply;
                    if ((apply = transformer.apply(advance.val)) != null) {
                        action.accept(apply);
                    }
                    else {
                        continue;
                    }
                }
                this.propagateCompletion();
            }
        }
    }
    
    static final class ForEachTransformedEntryTask<K, V, U> extends ConcurrentHashMap.BulkTask<K, V, Void>
    {
        final Function<Map.Entry<K, V>, ? extends U> transformer;
        final Consumer<? super U> action;
        
        ForEachTransformedEntryTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, Function<Map.Entry<K, V>, ? extends U> transformer, Consumer<? super U> action) {
            super(bulkTask, n, n2, n3, array);
            this.transformer = transformer;
            this.action = action;
        }
        
        @Override
        public final void compute() {
            Function<ConcurrentHashMap.Node, Object> transformer;
            Consumer<Object> action;
            if ((transformer = this.transformer) != null && (action = this.action) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    new ConcurrentHashMap.ForEachTransformedEntryTask(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, transformer, action).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    Object apply;
                    if ((apply = transformer.apply(advance)) != null) {
                        action.accept(apply);
                    }
                    else {
                        continue;
                    }
                }
                this.propagateCompletion();
            }
        }
    }
    
    static final class ForEachTransformedMappingTask<K, V, U> extends ConcurrentHashMap.BulkTask<K, V, Void>
    {
        final BiFunction<? super K, ? super V, ? extends U> transformer;
        final Consumer<? super U> action;
        
        ForEachTransformedMappingTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, BiFunction<? super K, ? super V, ? extends U> transformer, Consumer<? super U> action) {
            super(bulkTask, n, n2, n3, array);
            this.transformer = transformer;
            this.action = action;
        }
        
        @Override
        public final void compute() {
            BiFunction<ConcurrentHashMap.Node, ConcurrentHashMap.Node, Object> transformer;
            Consumer<Object> action;
            if ((transformer = this.transformer) != null && (action = this.action) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    new ConcurrentHashMap.ForEachTransformedMappingTask(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, transformer, action).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    Object apply;
                    if ((apply = transformer.apply(advance.key, advance.val)) != null) {
                        action.accept(apply);
                    }
                    else {
                        continue;
                    }
                }
                this.propagateCompletion();
            }
        }
    }
    
    static final class SearchKeysTask<K, V, U> extends ConcurrentHashMap.BulkTask<K, V, U>
    {
        final Function<? super K, ? extends U> searchFunction;
        final AtomicReference<U> result;
        
        SearchKeysTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, Function<? super K, ? extends U> searchFunction, AtomicReference<U> result) {
            super(bulkTask, n, n2, n3, array);
            this.searchFunction = searchFunction;
            this.result = result;
        }
        
        @Override
        public final U getRawResult() {
            return this.result.get();
        }
        
        @Override
        public final void compute() {
            Function<ConcurrentHashMap.Node, Object> searchFunction;
            AtomicReference result;
            if ((searchFunction = this.searchFunction) != null && (result = this.result) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    if (result.get() != null) {
                        return;
                    }
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    new ConcurrentHashMap.SearchKeysTask(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, searchFunction, result).fork();
                }
                while (result.get() == null) {
                    ConcurrentHashMap.Node<K, V> advance;
                    if ((advance = this.advance()) == null) {
                        this.propagateCompletion();
                        break;
                    }
                    Object apply;
                    if ((apply = searchFunction.apply(advance.key)) != null) {
                        if (result.compareAndSet(null, apply)) {
                            this.quietlyCompleteRoot();
                            break;
                        }
                        break;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
    }
    
    static final class SearchValuesTask<K, V, U> extends ConcurrentHashMap.BulkTask<K, V, U>
    {
        final Function<? super V, ? extends U> searchFunction;
        final AtomicReference<U> result;
        
        SearchValuesTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, Function<? super V, ? extends U> searchFunction, AtomicReference<U> result) {
            super(bulkTask, n, n2, n3, array);
            this.searchFunction = searchFunction;
            this.result = result;
        }
        
        @Override
        public final U getRawResult() {
            return this.result.get();
        }
        
        @Override
        public final void compute() {
            Function<ConcurrentHashMap.Node, Object> searchFunction;
            AtomicReference result;
            if ((searchFunction = this.searchFunction) != null && (result = this.result) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    if (result.get() != null) {
                        return;
                    }
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    new ConcurrentHashMap.SearchValuesTask(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, searchFunction, result).fork();
                }
                while (result.get() == null) {
                    ConcurrentHashMap.Node<K, V> advance;
                    if ((advance = this.advance()) == null) {
                        this.propagateCompletion();
                        break;
                    }
                    Object apply;
                    if ((apply = searchFunction.apply(advance.val)) != null) {
                        if (result.compareAndSet(null, apply)) {
                            this.quietlyCompleteRoot();
                            break;
                        }
                        break;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
    }
    
    static final class SearchEntriesTask<K, V, U> extends ConcurrentHashMap.BulkTask<K, V, U>
    {
        final Function<Map.Entry<K, V>, ? extends U> searchFunction;
        final AtomicReference<U> result;
        
        SearchEntriesTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, Function<Map.Entry<K, V>, ? extends U> searchFunction, AtomicReference<U> result) {
            super(bulkTask, n, n2, n3, array);
            this.searchFunction = searchFunction;
            this.result = result;
        }
        
        @Override
        public final U getRawResult() {
            return this.result.get();
        }
        
        @Override
        public final void compute() {
            Function<ConcurrentHashMap.Node, Object> searchFunction;
            AtomicReference result;
            if ((searchFunction = this.searchFunction) != null && (result = this.result) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    if (result.get() != null) {
                        return;
                    }
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    new ConcurrentHashMap.SearchEntriesTask(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, searchFunction, result).fork();
                }
                while (result.get() == null) {
                    ConcurrentHashMap.Node<K, V> advance;
                    if ((advance = this.advance()) == null) {
                        this.propagateCompletion();
                        break;
                    }
                    Object apply;
                    if ((apply = searchFunction.apply(advance)) != null) {
                        if (result.compareAndSet(null, apply)) {
                            this.quietlyCompleteRoot();
                        }
                        return;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
    }
    
    static final class SearchMappingsTask<K, V, U> extends ConcurrentHashMap.BulkTask<K, V, U>
    {
        final BiFunction<? super K, ? super V, ? extends U> searchFunction;
        final AtomicReference<U> result;
        
        SearchMappingsTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, BiFunction<? super K, ? super V, ? extends U> searchFunction, AtomicReference<U> result) {
            super(bulkTask, n, n2, n3, array);
            this.searchFunction = searchFunction;
            this.result = result;
        }
        
        @Override
        public final U getRawResult() {
            return this.result.get();
        }
        
        @Override
        public final void compute() {
            BiFunction<ConcurrentHashMap.Node, ConcurrentHashMap.Node, Object> searchFunction;
            AtomicReference result;
            if ((searchFunction = this.searchFunction) != null && (result = this.result) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    if (result.get() != null) {
                        return;
                    }
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    new ConcurrentHashMap.SearchMappingsTask(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, searchFunction, result).fork();
                }
                while (result.get() == null) {
                    ConcurrentHashMap.Node<K, V> advance;
                    if ((advance = this.advance()) == null) {
                        this.propagateCompletion();
                        break;
                    }
                    Object apply;
                    if ((apply = searchFunction.apply(advance.key, advance.val)) != null) {
                        if (result.compareAndSet(null, apply)) {
                            this.quietlyCompleteRoot();
                            break;
                        }
                        break;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
    }
    
    static final class ReduceKeysTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, K>
    {
        final BiFunction<? super K, ? super K, ? extends K> reducer;
        K result;
        ConcurrentHashMap.ReduceKeysTask<K, V> rights;
        ConcurrentHashMap.ReduceKeysTask<K, V> nextRight;
        
        ReduceKeysTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.ReduceKeysTask<K, V> nextRight, BiFunction<? super K, ? super K, ? extends K> reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.reducer = reducer;
        }
        
        @Override
        public final K getRawResult() {
            return this.result;
        }
        
        @Override
        public final void compute() {
            BiFunction<Object, Object, Object> reducer;
            if ((reducer = this.reducer) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.ReduceKeysTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, reducer)).fork();
                }
                Object result = null;
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    Object key = advance.key;
                    result = ((result == null) ? key : ((key == null) ? result : reducer.apply(result, key)));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.ReduceKeysTask<K, V> nextRight;
                    for (ConcurrentHashMap.ReduceKeysTask reduceKeysTask = (ConcurrentHashMap.ReduceKeysTask)countedCompleter, rights = reduceKeysTask.rights; rights != null; nextRight = rights.nextRight, reduceKeysTask.rights = nextRight, rights = nextRight) {
                        Object result2;
                        if ((result2 = rights.result) != null) {
                            Object result3;
                            reduceKeysTask.result = (((result3 = reduceKeysTask.result) == null) ? result2 : reducer.apply(result3, result2));
                        }
                    }
                }
            }
        }
    }
    
    static final class ReduceValuesTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, V>
    {
        final BiFunction<? super V, ? super V, ? extends V> reducer;
        V result;
        ConcurrentHashMap.ReduceValuesTask<K, V> rights;
        ConcurrentHashMap.ReduceValuesTask<K, V> nextRight;
        
        ReduceValuesTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.ReduceValuesTask<K, V> nextRight, BiFunction<? super V, ? super V, ? extends V> reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.reducer = reducer;
        }
        
        @Override
        public final V getRawResult() {
            return this.result;
        }
        
        @Override
        public final void compute() {
            BiFunction<Object, Object, Object> reducer;
            if ((reducer = this.reducer) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.ReduceValuesTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, reducer)).fork();
                }
                Object result = null;
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    Object val = advance.val;
                    result = ((result == null) ? val : reducer.apply(result, val));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.ReduceValuesTask<K, V> nextRight;
                    for (ConcurrentHashMap.ReduceValuesTask reduceValuesTask = (ConcurrentHashMap.ReduceValuesTask)countedCompleter, rights = reduceValuesTask.rights; rights != null; nextRight = rights.nextRight, reduceValuesTask.rights = nextRight, rights = nextRight) {
                        Object result2;
                        if ((result2 = rights.result) != null) {
                            Object result3;
                            reduceValuesTask.result = (((result3 = reduceValuesTask.result) == null) ? result2 : reducer.apply(result3, result2));
                        }
                    }
                }
            }
        }
    }
    
    static final class ReduceEntriesTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Map.Entry<K, V>>
    {
        final BiFunction<Map.Entry<K, V>, Map.Entry<K, V>, ? extends Map.Entry<K, V>> reducer;
        Map.Entry<K, V> result;
        ConcurrentHashMap.ReduceEntriesTask<K, V> rights;
        ConcurrentHashMap.ReduceEntriesTask<K, V> nextRight;
        
        ReduceEntriesTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.ReduceEntriesTask<K, V> nextRight, BiFunction<Map.Entry<K, V>, Map.Entry<K, V>, ? extends Map.Entry<K, V>> reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.reducer = reducer;
        }
        
        @Override
        public final Map.Entry<K, V> getRawResult() {
            return this.result;
        }
        
        @Override
        public final void compute() {
            BiFunction<Map.Entry, Object, Object> reducer;
            if ((reducer = this.reducer) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.ReduceEntriesTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, reducer)).fork();
                }
                Map.Entry<Object, Object> result = null;
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    result = ((result == null) ? advance : ((Map.Entry)reducer.apply(result, advance)));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.ReduceEntriesTask<K, V> nextRight;
                    for (ConcurrentHashMap.ReduceEntriesTask reduceEntriesTask = (ConcurrentHashMap.ReduceEntriesTask)countedCompleter, rights = reduceEntriesTask.rights; rights != null; nextRight = rights.nextRight, reduceEntriesTask.rights = nextRight, rights = nextRight) {
                        Map.Entry<Object, Object> result2;
                        if ((result2 = rights.result) != null) {
                            Map.Entry<Object, Object> result3;
                            reduceEntriesTask.result = (((result3 = reduceEntriesTask.result) == null) ? result2 : ((Map.Entry)reducer.apply(result3, result2)));
                        }
                    }
                }
            }
        }
    }
    
    static final class MapReduceKeysTask<K, V, U> extends ConcurrentHashMap.BulkTask<K, V, U>
    {
        final Function<? super K, ? extends U> transformer;
        final BiFunction<? super U, ? super U, ? extends U> reducer;
        U result;
        ConcurrentHashMap.MapReduceKeysTask<K, V, U> rights;
        ConcurrentHashMap.MapReduceKeysTask<K, V, U> nextRight;
        
        MapReduceKeysTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceKeysTask<K, V, U> nextRight, Function<? super K, ? extends U> transformer, BiFunction<? super U, ? super U, ? extends U> reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.reducer = reducer;
        }
        
        @Override
        public final U getRawResult() {
            return this.result;
        }
        
        @Override
        public final void compute() {
            Function<ConcurrentHashMap.Node, Object> transformer;
            BiFunction<Object, Object, Object> reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceKeysTask<K, V, U>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, reducer)).fork();
                }
                Object result = null;
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    Object apply;
                    if ((apply = transformer.apply(advance.key)) != null) {
                        result = ((result == null) ? apply : reducer.apply(result, apply));
                    }
                    else {
                        continue;
                    }
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceKeysTask<K, V, U> nextRight;
                    for (ConcurrentHashMap.MapReduceKeysTask mapReduceKeysTask = (ConcurrentHashMap.MapReduceKeysTask)countedCompleter, rights = mapReduceKeysTask.rights; rights != null; nextRight = rights.nextRight, mapReduceKeysTask.rights = nextRight, rights = nextRight) {
                        Object result2;
                        if ((result2 = rights.result) != null) {
                            Object result3;
                            mapReduceKeysTask.result = (((result3 = mapReduceKeysTask.result) == null) ? result2 : reducer.apply(result3, result2));
                        }
                    }
                }
            }
        }
    }
    
    static final class MapReduceValuesTask<K, V, U> extends ConcurrentHashMap.BulkTask<K, V, U>
    {
        final Function<? super V, ? extends U> transformer;
        final BiFunction<? super U, ? super U, ? extends U> reducer;
        U result;
        ConcurrentHashMap.MapReduceValuesTask<K, V, U> rights;
        ConcurrentHashMap.MapReduceValuesTask<K, V, U> nextRight;
        
        MapReduceValuesTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceValuesTask<K, V, U> nextRight, Function<? super V, ? extends U> transformer, BiFunction<? super U, ? super U, ? extends U> reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.reducer = reducer;
        }
        
        @Override
        public final U getRawResult() {
            return this.result;
        }
        
        @Override
        public final void compute() {
            Function<ConcurrentHashMap.Node, Object> transformer;
            BiFunction<Object, Object, Object> reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceValuesTask<K, V, U>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, reducer)).fork();
                }
                Object result = null;
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    Object apply;
                    if ((apply = transformer.apply(advance.val)) != null) {
                        result = ((result == null) ? apply : reducer.apply(result, apply));
                    }
                    else {
                        continue;
                    }
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceValuesTask<K, V, U> nextRight;
                    for (ConcurrentHashMap.MapReduceValuesTask mapReduceValuesTask = (ConcurrentHashMap.MapReduceValuesTask)countedCompleter, rights = mapReduceValuesTask.rights; rights != null; nextRight = rights.nextRight, mapReduceValuesTask.rights = nextRight, rights = nextRight) {
                        Object result2;
                        if ((result2 = rights.result) != null) {
                            Object result3;
                            mapReduceValuesTask.result = (((result3 = mapReduceValuesTask.result) == null) ? result2 : reducer.apply(result3, result2));
                        }
                    }
                }
            }
        }
    }
    
    static final class MapReduceEntriesTask<K, V, U> extends ConcurrentHashMap.BulkTask<K, V, U>
    {
        final Function<Map.Entry<K, V>, ? extends U> transformer;
        final BiFunction<? super U, ? super U, ? extends U> reducer;
        U result;
        ConcurrentHashMap.MapReduceEntriesTask<K, V, U> rights;
        ConcurrentHashMap.MapReduceEntriesTask<K, V, U> nextRight;
        
        MapReduceEntriesTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceEntriesTask<K, V, U> nextRight, Function<Map.Entry<K, V>, ? extends U> transformer, BiFunction<? super U, ? super U, ? extends U> reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.reducer = reducer;
        }
        
        @Override
        public final U getRawResult() {
            return this.result;
        }
        
        @Override
        public final void compute() {
            Function<ConcurrentHashMap.Node, Object> transformer;
            BiFunction<Object, Object, Object> reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceEntriesTask<K, V, U>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, reducer)).fork();
                }
                Object result = null;
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    Object apply;
                    if ((apply = transformer.apply(advance)) != null) {
                        result = ((result == null) ? apply : reducer.apply(result, apply));
                    }
                    else {
                        continue;
                    }
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceEntriesTask<K, V, U> nextRight;
                    for (ConcurrentHashMap.MapReduceEntriesTask mapReduceEntriesTask = (ConcurrentHashMap.MapReduceEntriesTask)countedCompleter, rights = mapReduceEntriesTask.rights; rights != null; nextRight = rights.nextRight, mapReduceEntriesTask.rights = nextRight, rights = nextRight) {
                        Object result2;
                        if ((result2 = rights.result) != null) {
                            Object result3;
                            mapReduceEntriesTask.result = (((result3 = mapReduceEntriesTask.result) == null) ? result2 : reducer.apply(result3, result2));
                        }
                    }
                }
            }
        }
    }
    
    static final class MapReduceMappingsTask<K, V, U> extends ConcurrentHashMap.BulkTask<K, V, U>
    {
        final BiFunction<? super K, ? super V, ? extends U> transformer;
        final BiFunction<? super U, ? super U, ? extends U> reducer;
        U result;
        ConcurrentHashMap.MapReduceMappingsTask<K, V, U> rights;
        ConcurrentHashMap.MapReduceMappingsTask<K, V, U> nextRight;
        
        MapReduceMappingsTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceMappingsTask<K, V, U> nextRight, BiFunction<? super K, ? super V, ? extends U> transformer, BiFunction<? super U, ? super U, ? extends U> reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.reducer = reducer;
        }
        
        @Override
        public final U getRawResult() {
            return this.result;
        }
        
        @Override
        public final void compute() {
            BiFunction<ConcurrentHashMap.Node, ConcurrentHashMap.Node, Object> transformer;
            BiFunction<Object, Object, Object> reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceMappingsTask<K, V, U>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, reducer)).fork();
                }
                Object result = null;
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    Object apply;
                    if ((apply = transformer.apply(advance.key, advance.val)) != null) {
                        result = ((result == null) ? apply : reducer.apply(result, apply));
                    }
                    else {
                        continue;
                    }
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceMappingsTask<K, V, U> nextRight;
                    for (ConcurrentHashMap.MapReduceMappingsTask mapReduceMappingsTask = (ConcurrentHashMap.MapReduceMappingsTask)countedCompleter, rights = mapReduceMappingsTask.rights; rights != null; nextRight = rights.nextRight, mapReduceMappingsTask.rights = nextRight, rights = nextRight) {
                        Object result2;
                        if ((result2 = rights.result) != null) {
                            Object result3;
                            mapReduceMappingsTask.result = (((result3 = mapReduceMappingsTask.result) == null) ? result2 : reducer.apply(result3, result2));
                        }
                    }
                }
            }
        }
    }
    
    static final class MapReduceKeysToDoubleTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Double>
    {
        final ToDoubleFunction<? super K> transformer;
        final DoubleBinaryOperator reducer;
        final double basis;
        double result;
        ConcurrentHashMap.MapReduceKeysToDoubleTask<K, V> rights;
        ConcurrentHashMap.MapReduceKeysToDoubleTask<K, V> nextRight;
        
        MapReduceKeysToDoubleTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceKeysToDoubleTask<K, V> nextRight, ToDoubleFunction<? super K> transformer, double basis, DoubleBinaryOperator reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.basis = basis;
            this.reducer = reducer;
        }
        
        @Override
        public final Double getRawResult() {
            return Double.valueOf(this.result);
        }
        
        @Override
        public final void compute() {
            ToDoubleFunction<ConcurrentHashMap.Node> transformer;
            DoubleBinaryOperator reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                double result = this.basis;
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceKeysToDoubleTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, result, reducer)).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    result = reducer.applyAsDouble(result, transformer.applyAsDouble(advance.key));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceKeysToDoubleTask<Object, Object> mapReduceKeysToDoubleTask = (ConcurrentHashMap.MapReduceKeysToDoubleTask)countedCompleter;
                    ConcurrentHashMap.MapReduceKeysToDoubleTask<K, V> nextRight;
                    for (ConcurrentHashMap.MapReduceKeysToDoubleTask rights = mapReduceKeysToDoubleTask.rights; rights != null; rights = nextRight) {
                        mapReduceKeysToDoubleTask.result = reducer.applyAsDouble(mapReduceKeysToDoubleTask.result, rights.result);
                        nextRight = rights.nextRight;
                        mapReduceKeysToDoubleTask.rights = nextRight;
                    }
                }
            }
        }
    }
    
    static final class MapReduceValuesToDoubleTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Double>
    {
        final ToDoubleFunction<? super V> transformer;
        final DoubleBinaryOperator reducer;
        final double basis;
        double result;
        ConcurrentHashMap.MapReduceValuesToDoubleTask<K, V> rights;
        ConcurrentHashMap.MapReduceValuesToDoubleTask<K, V> nextRight;
        
        MapReduceValuesToDoubleTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceValuesToDoubleTask<K, V> nextRight, ToDoubleFunction<? super V> transformer, double basis, DoubleBinaryOperator reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.basis = basis;
            this.reducer = reducer;
        }
        
        @Override
        public final Double getRawResult() {
            return Double.valueOf(this.result);
        }
        
        @Override
        public final void compute() {
            ToDoubleFunction<ConcurrentHashMap.Node> transformer;
            DoubleBinaryOperator reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                double result = this.basis;
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceValuesToDoubleTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, result, reducer)).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    result = reducer.applyAsDouble(result, transformer.applyAsDouble(advance.val));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceValuesToDoubleTask<Object, Object> mapReduceValuesToDoubleTask = (ConcurrentHashMap.MapReduceValuesToDoubleTask)countedCompleter;
                    ConcurrentHashMap.MapReduceValuesToDoubleTask<K, V> nextRight;
                    for (ConcurrentHashMap.MapReduceValuesToDoubleTask rights = mapReduceValuesToDoubleTask.rights; rights != null; rights = nextRight) {
                        mapReduceValuesToDoubleTask.result = reducer.applyAsDouble(mapReduceValuesToDoubleTask.result, rights.result);
                        nextRight = rights.nextRight;
                        mapReduceValuesToDoubleTask.rights = nextRight;
                    }
                }
            }
        }
    }
    
    static final class MapReduceEntriesToDoubleTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Double>
    {
        final ToDoubleFunction<Map.Entry<K, V>> transformer;
        final DoubleBinaryOperator reducer;
        final double basis;
        double result;
        ConcurrentHashMap.MapReduceEntriesToDoubleTask<K, V> rights;
        ConcurrentHashMap.MapReduceEntriesToDoubleTask<K, V> nextRight;
        
        MapReduceEntriesToDoubleTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceEntriesToDoubleTask<K, V> nextRight, ToDoubleFunction<Map.Entry<K, V>> transformer, double basis, DoubleBinaryOperator reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.basis = basis;
            this.reducer = reducer;
        }
        
        @Override
        public final Double getRawResult() {
            return Double.valueOf(this.result);
        }
        
        @Override
        public final void compute() {
            ToDoubleFunction<ConcurrentHashMap.Node> transformer;
            DoubleBinaryOperator reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                double result = this.basis;
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceEntriesToDoubleTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, result, reducer)).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    result = reducer.applyAsDouble(result, transformer.applyAsDouble(advance));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceEntriesToDoubleTask<Object, Object> mapReduceEntriesToDoubleTask = (ConcurrentHashMap.MapReduceEntriesToDoubleTask)countedCompleter;
                    ConcurrentHashMap.MapReduceEntriesToDoubleTask<K, V> nextRight;
                    for (ConcurrentHashMap.MapReduceEntriesToDoubleTask rights = mapReduceEntriesToDoubleTask.rights; rights != null; rights = nextRight) {
                        mapReduceEntriesToDoubleTask.result = reducer.applyAsDouble(mapReduceEntriesToDoubleTask.result, rights.result);
                        nextRight = rights.nextRight;
                        mapReduceEntriesToDoubleTask.rights = nextRight;
                    }
                }
            }
        }
    }
    
    static final class MapReduceMappingsToDoubleTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Double>
    {
        final ToDoubleBiFunction<? super K, ? super V> transformer;
        final DoubleBinaryOperator reducer;
        final double basis;
        double result;
        ConcurrentHashMap.MapReduceMappingsToDoubleTask<K, V> rights;
        ConcurrentHashMap.MapReduceMappingsToDoubleTask<K, V> nextRight;
        
        MapReduceMappingsToDoubleTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceMappingsToDoubleTask<K, V> nextRight, ToDoubleBiFunction<? super K, ? super V> transformer, double basis, DoubleBinaryOperator reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.basis = basis;
            this.reducer = reducer;
        }
        
        @Override
        public final Double getRawResult() {
            return Double.valueOf(this.result);
        }
        
        @Override
        public final void compute() {
            ToDoubleBiFunction<ConcurrentHashMap.Node, ConcurrentHashMap.Node> transformer;
            DoubleBinaryOperator reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                double result = this.basis;
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceMappingsToDoubleTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, result, reducer)).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    result = reducer.applyAsDouble(result, transformer.applyAsDouble(advance.key, advance.val));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceMappingsToDoubleTask<Object, Object> mapReduceMappingsToDoubleTask = (ConcurrentHashMap.MapReduceMappingsToDoubleTask)countedCompleter;
                    ConcurrentHashMap.MapReduceMappingsToDoubleTask<K, V> nextRight;
                    for (ConcurrentHashMap.MapReduceMappingsToDoubleTask rights = mapReduceMappingsToDoubleTask.rights; rights != null; rights = nextRight) {
                        mapReduceMappingsToDoubleTask.result = reducer.applyAsDouble(mapReduceMappingsToDoubleTask.result, rights.result);
                        nextRight = rights.nextRight;
                        mapReduceMappingsToDoubleTask.rights = nextRight;
                    }
                }
            }
        }
    }
    
    static final class MapReduceKeysToLongTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Long>
    {
        final ToLongFunction<? super K> transformer;
        final LongBinaryOperator reducer;
        final long basis;
        long result;
        ConcurrentHashMap.MapReduceKeysToLongTask<K, V> rights;
        ConcurrentHashMap.MapReduceKeysToLongTask<K, V> nextRight;
        
        MapReduceKeysToLongTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceKeysToLongTask<K, V> nextRight, ToLongFunction<? super K> transformer, long basis, LongBinaryOperator reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.basis = basis;
            this.reducer = reducer;
        }
        
        @Override
        public final Long getRawResult() {
            return Long.valueOf(this.result);
        }
        
        @Override
        public final void compute() {
            ToLongFunction<ConcurrentHashMap.Node> transformer;
            LongBinaryOperator reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                long result = this.basis;
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceKeysToLongTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, result, reducer)).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    result = reducer.applyAsLong(result, transformer.applyAsLong(advance.key));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceKeysToLongTask<Object, Object> mapReduceKeysToLongTask = (ConcurrentHashMap.MapReduceKeysToLongTask)countedCompleter;
                    ConcurrentHashMap.MapReduceKeysToLongTask<K, V> nextRight;
                    for (ConcurrentHashMap.MapReduceKeysToLongTask rights = mapReduceKeysToLongTask.rights; rights != null; rights = nextRight) {
                        mapReduceKeysToLongTask.result = reducer.applyAsLong(mapReduceKeysToLongTask.result, rights.result);
                        nextRight = rights.nextRight;
                        mapReduceKeysToLongTask.rights = nextRight;
                    }
                }
            }
        }
    }
    
    static final class MapReduceValuesToLongTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Long>
    {
        final ToLongFunction<? super V> transformer;
        final LongBinaryOperator reducer;
        final long basis;
        long result;
        ConcurrentHashMap.MapReduceValuesToLongTask<K, V> rights;
        ConcurrentHashMap.MapReduceValuesToLongTask<K, V> nextRight;
        
        MapReduceValuesToLongTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceValuesToLongTask<K, V> nextRight, ToLongFunction<? super V> transformer, long basis, LongBinaryOperator reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.basis = basis;
            this.reducer = reducer;
        }
        
        @Override
        public final Long getRawResult() {
            return Long.valueOf(this.result);
        }
        
        @Override
        public final void compute() {
            ToLongFunction<ConcurrentHashMap.Node> transformer;
            LongBinaryOperator reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                long result = this.basis;
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceValuesToLongTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, result, reducer)).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    result = reducer.applyAsLong(result, transformer.applyAsLong(advance.val));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceValuesToLongTask<Object, Object> mapReduceValuesToLongTask = (ConcurrentHashMap.MapReduceValuesToLongTask)countedCompleter;
                    ConcurrentHashMap.MapReduceValuesToLongTask<K, V> nextRight;
                    for (ConcurrentHashMap.MapReduceValuesToLongTask rights = mapReduceValuesToLongTask.rights; rights != null; rights = nextRight) {
                        mapReduceValuesToLongTask.result = reducer.applyAsLong(mapReduceValuesToLongTask.result, rights.result);
                        nextRight = rights.nextRight;
                        mapReduceValuesToLongTask.rights = nextRight;
                    }
                }
            }
        }
    }
    
    static final class MapReduceEntriesToLongTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Long>
    {
        final ToLongFunction<Map.Entry<K, V>> transformer;
        final LongBinaryOperator reducer;
        final long basis;
        long result;
        ConcurrentHashMap.MapReduceEntriesToLongTask<K, V> rights;
        ConcurrentHashMap.MapReduceEntriesToLongTask<K, V> nextRight;
        
        MapReduceEntriesToLongTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceEntriesToLongTask<K, V> nextRight, ToLongFunction<Map.Entry<K, V>> transformer, long basis, LongBinaryOperator reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.basis = basis;
            this.reducer = reducer;
        }
        
        @Override
        public final Long getRawResult() {
            return Long.valueOf(this.result);
        }
        
        @Override
        public final void compute() {
            ToLongFunction<ConcurrentHashMap.Node> transformer;
            LongBinaryOperator reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                long result = this.basis;
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceEntriesToLongTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, result, reducer)).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    result = reducer.applyAsLong(result, transformer.applyAsLong(advance));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceEntriesToLongTask<Object, Object> mapReduceEntriesToLongTask = (ConcurrentHashMap.MapReduceEntriesToLongTask)countedCompleter;
                    ConcurrentHashMap.MapReduceEntriesToLongTask<K, V> nextRight;
                    for (ConcurrentHashMap.MapReduceEntriesToLongTask rights = mapReduceEntriesToLongTask.rights; rights != null; rights = nextRight) {
                        mapReduceEntriesToLongTask.result = reducer.applyAsLong(mapReduceEntriesToLongTask.result, rights.result);
                        nextRight = rights.nextRight;
                        mapReduceEntriesToLongTask.rights = nextRight;
                    }
                }
            }
        }
    }
    
    static final class MapReduceMappingsToLongTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Long>
    {
        final ToLongBiFunction<? super K, ? super V> transformer;
        final LongBinaryOperator reducer;
        final long basis;
        long result;
        ConcurrentHashMap.MapReduceMappingsToLongTask<K, V> rights;
        ConcurrentHashMap.MapReduceMappingsToLongTask<K, V> nextRight;
        
        MapReduceMappingsToLongTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceMappingsToLongTask<K, V> nextRight, ToLongBiFunction<? super K, ? super V> transformer, long basis, LongBinaryOperator reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.basis = basis;
            this.reducer = reducer;
        }
        
        @Override
        public final Long getRawResult() {
            return Long.valueOf(this.result);
        }
        
        @Override
        public final void compute() {
            ToLongBiFunction<ConcurrentHashMap.Node, ConcurrentHashMap.Node> transformer;
            LongBinaryOperator reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                long result = this.basis;
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceMappingsToLongTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, result, reducer)).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    result = reducer.applyAsLong(result, transformer.applyAsLong(advance.key, advance.val));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceMappingsToLongTask<Object, Object> mapReduceMappingsToLongTask = (ConcurrentHashMap.MapReduceMappingsToLongTask)countedCompleter;
                    ConcurrentHashMap.MapReduceMappingsToLongTask<K, V> nextRight;
                    for (ConcurrentHashMap.MapReduceMappingsToLongTask rights = mapReduceMappingsToLongTask.rights; rights != null; rights = nextRight) {
                        mapReduceMappingsToLongTask.result = reducer.applyAsLong(mapReduceMappingsToLongTask.result, rights.result);
                        nextRight = rights.nextRight;
                        mapReduceMappingsToLongTask.rights = nextRight;
                    }
                }
            }
        }
    }
    
    static final class MapReduceKeysToIntTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Integer>
    {
        final ToIntFunction<? super K> transformer;
        final IntBinaryOperator reducer;
        final int basis;
        int result;
        ConcurrentHashMap.MapReduceKeysToIntTask<K, V> rights;
        ConcurrentHashMap.MapReduceKeysToIntTask<K, V> nextRight;
        
        MapReduceKeysToIntTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceKeysToIntTask<K, V> nextRight, ToIntFunction<? super K> transformer, int basis, IntBinaryOperator reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.basis = basis;
            this.reducer = reducer;
        }
        
        @Override
        public final Integer getRawResult() {
            return Integer.valueOf(this.result);
        }
        
        @Override
        public final void compute() {
            ToIntFunction<ConcurrentHashMap.Node> transformer;
            IntBinaryOperator reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                int result = this.basis;
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceKeysToIntTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, result, reducer)).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    result = reducer.applyAsInt(result, transformer.applyAsInt(advance.key));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceKeysToIntTask<Object, Object> mapReduceKeysToIntTask = (ConcurrentHashMap.MapReduceKeysToIntTask)countedCompleter;
                    ConcurrentHashMap.MapReduceKeysToIntTask<K, V> nextRight;
                    for (ConcurrentHashMap.MapReduceKeysToIntTask rights = mapReduceKeysToIntTask.rights; rights != null; rights = nextRight) {
                        mapReduceKeysToIntTask.result = reducer.applyAsInt(mapReduceKeysToIntTask.result, rights.result);
                        nextRight = rights.nextRight;
                        mapReduceKeysToIntTask.rights = nextRight;
                    }
                }
            }
        }
    }
    
    static final class MapReduceValuesToIntTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Integer>
    {
        final ToIntFunction<? super V> transformer;
        final IntBinaryOperator reducer;
        final int basis;
        int result;
        ConcurrentHashMap.MapReduceValuesToIntTask<K, V> rights;
        ConcurrentHashMap.MapReduceValuesToIntTask<K, V> nextRight;
        
        MapReduceValuesToIntTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceValuesToIntTask<K, V> nextRight, ToIntFunction<? super V> transformer, int basis, IntBinaryOperator reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.basis = basis;
            this.reducer = reducer;
        }
        
        @Override
        public final Integer getRawResult() {
            return Integer.valueOf(this.result);
        }
        
        @Override
        public final void compute() {
            ToIntFunction<ConcurrentHashMap.Node> transformer;
            IntBinaryOperator reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                int result = this.basis;
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceValuesToIntTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, result, reducer)).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    result = reducer.applyAsInt(result, transformer.applyAsInt(advance.val));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceValuesToIntTask<Object, Object> mapReduceValuesToIntTask = (ConcurrentHashMap.MapReduceValuesToIntTask)countedCompleter;
                    ConcurrentHashMap.MapReduceValuesToIntTask<K, V> nextRight;
                    for (ConcurrentHashMap.MapReduceValuesToIntTask rights = mapReduceValuesToIntTask.rights; rights != null; rights = nextRight) {
                        mapReduceValuesToIntTask.result = reducer.applyAsInt(mapReduceValuesToIntTask.result, rights.result);
                        nextRight = rights.nextRight;
                        mapReduceValuesToIntTask.rights = nextRight;
                    }
                }
            }
        }
    }
    
    static final class MapReduceEntriesToIntTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Integer>
    {
        final ToIntFunction<Map.Entry<K, V>> transformer;
        final IntBinaryOperator reducer;
        final int basis;
        int result;
        ConcurrentHashMap.MapReduceEntriesToIntTask<K, V> rights;
        ConcurrentHashMap.MapReduceEntriesToIntTask<K, V> nextRight;
        
        MapReduceEntriesToIntTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceEntriesToIntTask<K, V> nextRight, ToIntFunction<Map.Entry<K, V>> transformer, int basis, IntBinaryOperator reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.basis = basis;
            this.reducer = reducer;
        }
        
        @Override
        public final Integer getRawResult() {
            return Integer.valueOf(this.result);
        }
        
        @Override
        public final void compute() {
            ToIntFunction<ConcurrentHashMap.Node> transformer;
            IntBinaryOperator reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                int result = this.basis;
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceEntriesToIntTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, result, reducer)).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    result = reducer.applyAsInt(result, transformer.applyAsInt(advance));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceEntriesToIntTask<Object, Object> mapReduceEntriesToIntTask = (ConcurrentHashMap.MapReduceEntriesToIntTask)countedCompleter;
                    ConcurrentHashMap.MapReduceEntriesToIntTask<K, V> nextRight;
                    for (ConcurrentHashMap.MapReduceEntriesToIntTask rights = mapReduceEntriesToIntTask.rights; rights != null; rights = nextRight) {
                        mapReduceEntriesToIntTask.result = reducer.applyAsInt(mapReduceEntriesToIntTask.result, rights.result);
                        nextRight = rights.nextRight;
                        mapReduceEntriesToIntTask.rights = nextRight;
                    }
                }
            }
        }
    }
    
    static final class MapReduceMappingsToIntTask<K, V> extends ConcurrentHashMap.BulkTask<K, V, Integer>
    {
        final ToIntBiFunction<? super K, ? super V> transformer;
        final IntBinaryOperator reducer;
        final int basis;
        int result;
        ConcurrentHashMap.MapReduceMappingsToIntTask<K, V> rights;
        ConcurrentHashMap.MapReduceMappingsToIntTask<K, V> nextRight;
        
        MapReduceMappingsToIntTask(ConcurrentHashMap.BulkTask<K, V, ?> bulkTask, int n, int n2, int n3, ConcurrentHashMap.Node<K, V>[] array, ConcurrentHashMap.MapReduceMappingsToIntTask<K, V> nextRight, ToIntBiFunction<? super K, ? super V> transformer, int basis, IntBinaryOperator reducer) {
            super(bulkTask, n, n2, n3, array);
            this.nextRight = nextRight;
            this.transformer = transformer;
            this.basis = basis;
            this.reducer = reducer;
        }
        
        @Override
        public final Integer getRawResult() {
            return Integer.valueOf(this.result);
        }
        
        @Override
        public final void compute() {
            ToIntBiFunction<ConcurrentHashMap.Node, ConcurrentHashMap.Node> transformer;
            IntBinaryOperator reducer;
            if ((transformer = this.transformer) != null && (reducer = this.reducer) != null) {
                int result = this.basis;
                int baseIndex = this.baseIndex;
                int baseLimit2;
                int baseLimit;
                while (this.batch > 0 && (baseLimit = (baseLimit2 = this.baseLimit) + baseIndex >>> 1) > baseIndex) {
                    this.addToPendingCount(1);
                    int batch = this.batch >>> 1;
                    this.batch = batch;
                    (this.rights = new ConcurrentHashMap.MapReduceMappingsToIntTask<K, V>(this, batch, this.baseLimit = baseLimit, baseLimit2, this.tab, this.rights, transformer, result, reducer)).fork();
                }
                ConcurrentHashMap.Node<K, V> advance;
                while ((advance = this.advance()) != null) {
                    result = reducer.applyAsInt(result, transformer.applyAsInt(advance.key, advance.val));
                }
                this.result = result;
                for (CountedCompleter countedCompleter = this.firstComplete(); countedCompleter != null; countedCompleter = countedCompleter.nextComplete()) {
                    ConcurrentHashMap.MapReduceMappingsToIntTask<Object, Object> mapReduceMappingsToIntTask = (ConcurrentHashMap.MapReduceMappingsToIntTask)countedCompleter;
                    ConcurrentHashMap.MapReduceMappingsToIntTask<K, V> nextRight;
                    for (ConcurrentHashMap.MapReduceMappingsToIntTask rights = mapReduceMappingsToIntTask.rights; rights != null; rights = nextRight) {
                        mapReduceMappingsToIntTask.result = reducer.applyAsInt(mapReduceMappingsToIntTask.result, rights.result);
                        nextRight = rights.nextRight;
                        mapReduceMappingsToIntTask.rights = nextRight;
                    }
                }
            }
        }
    }
}
